from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, VariantRule, RevisionRule, Awb0ProductContextInfo
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ConfigRuleInput(TcBaseObj):
    """
    The ConfigRuleInput contains Awb0ProductContextInfo, pagination, type to fetch and optionally search criteria to
    filter the result.
    
    :var productContext: Awb0ProductContextInfo containing current product and configuration information.
    :var startIndex: Start index for pagination.
    :var maxToLoad: Maximum number of RevisionRule or VariantRule to be returned. Return everything if 'maxToLoad' is
    -1.
    :var typeToFetch: Internal name of the type to be returned. Valid values are "RevisionRule" and "VariantRule". The
    input is case sensitive.
    :var fetchOnlyPreferredConfiguration: If this is true then return only preferred effectivity information or
    preferred VariantRule owning objects for current RevisionRule.
    :var searchCriteria: Object Name to filter the response objects.
    :var requestPref: Map of preference name and value pairs (string/string).
    """
    productContext: Awb0ProductContextInfo = None
    startIndex: int = 0
    maxToLoad: int = 0
    typeToFetch: str = ''
    fetchOnlyPreferredConfiguration: bool = False
    searchCriteria: str = ''
    requestPref: RequestPreference = None


@dataclass
class ConfigRuleResponse(TcBaseObj):
    """
    ConfigRuleResponse contains RevisionRule and VariantRule information.
    
    :var marker: Number of preferred RevisionRule or VariantRule. Preferred RevisionRule or VariantRule are the one
    which are indexed through structure indexing. The default value is -1.
    :var endIndex: End index of page.
    :var totalFound: Total numbers of RevisionRule or VariantRule found.
    :var globalRevisionRule: Global RevisionRule based on preference "TC_config_rule_name".
    :var revisionRules: List of unique RevisionRule.
    :var preferredEffectivityInfo: Fetch the preferred effectivity information based on prdouct and RevisionRule only
    when 'fetchOnlyPreferredConfiguration' is true.
    :var addOpenObjAsPreferredEndItem: This flag is true. If none of the preferred end item revision or preferred
    VariantRule owninig object points to root.
    :var variantRules: List of VariantRule based on RevisionRule and owning object for VariantRule.
    :var preferredVarRuleOwningObjects: Fetch the list of owning object based on RevisionRule and product only when
    'fetchOnlyPreferredConfiguration' is true.
    :var serviceData: The Service Data.
    """
    marker: int = 0
    endIndex: int = 0
    totalFound: int = 0
    globalRevisionRule: RevisionRule = None
    revisionRules: List[RevisionRule] = ()
    preferredEffectivityInfo: EffectivityInfo = None
    addOpenObjAsPreferredEndItem: bool = False
    variantRules: List[VariantRule] = ()
    preferredVarRuleOwningObjects: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class EffectivityInfo(TcBaseObj):
    """
    EffectivityInfo contains list of effectivity units, effectivity end items and effectivity dates.
    
    :var effectivityUnits: List of effectivity units.
    :var effectivityEndItems: List of effectivity end items.
    :var effectivityDates: List of effectivity dates.
    """
    effectivityUnits: List[int] = ()
    effectivityEndItems: List[BusinessObject] = ()
    effectivityDates: List[datetime] = ()


"""
RequestPreference is a map of preference name and value.
"""
RequestPreference = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.Reports._2008_12.CrfReports import GenerateReportsCriteria, GenerateReportsResponse
from typing import List
from tcsoa.utils import TcService


class CrfReportsService(TcService):

    @classmethod
    def generateReports(cls, inputs: List[GenerateReportsCriteria]) -> GenerateReportsResponse:
        """
        Generates reports (Summary Report/Custom Report/Item Report) using the specified criteria and the selected
        report style. The report will be displayed by the selected report style at the end. If no report style is
        selected, then the report will be displayed in xml file to the end user. If user would like to save the report
        as a Dataset, it will use the provided Dataset name to save the report to Teamcenter. After the report is
        generated, the report file will be uploaded to the transient volumes, user can get it from the transient
        volumes. Multiple reports generation is not supported currently.
        
        Use cases:
        User can generate one report (Summary Report/Custom Report/Item Report) by selecting one report definition and
        then inputs criteria for the report query, selects one report style, and inputs the Dataset name for the report
        if user would like to save the report as a Dataset in Teamcenter.
        """
        return cls.execute_soa_method(
            method_name='generateReports',
            library='Reports',
            service_date='2008_12',
            service_name='CrfReports',
            params={'inputs': inputs},
            response_cls=GenerateReportsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, VariantRule
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetProductStructureIdResponse(TcBaseObj):
    """
    This structure contains the information about the structure ID used to determine the product structure information
    from the catalogue of product structure information maintained in the VDS. The configuration that need to be
    applied on that product structure to configure it is also returned via this response.
    
    :var psIdInfos: A list of ProductStructureIdInfo containing the id of  product structure information and the
    configuration that need to be applied.
    :var serviceData: ServiceData by which the partial errors are communicated to the client.
    """
    psIdInfos: List[ProductStructureIdInfo] = ()
    serviceData: ServiceData = None


@dataclass
class ProductAndConfigInfo(TcBaseObj):
    """
    This structure contains product and the associated configuration for which the product structure information id is
    to be determined. The configuration information can either be passed in a detailed format or as recipe object.
    
    :var product: The product which is indexed. IThe supported object types can either be a  BOMViewRevision or
    Cpd0CollaborativeDesign.
    :var revisionRule: The RevisionRule configuration applied on the product structure.
    :var effectivityInfos: A list of effectivity information for which the product structure was configured.
    :var variantInfos: A list of VariantRule objects applied on the product structure.
    """
    product: BusinessObject = None
    revisionRule: BusinessObject = None
    effectivityInfos: List[EffectivityInfo] = ()
    variantInfos: List[VariantRuleInfo] = ()


@dataclass
class ProductStructureIdInfo(TcBaseObj):
    """
    This structure contains the id of the product structure and the configuration information that need to be applied
    to the product structure.
    
    :var clientId: A clientId passed by the client in order to map the input to the corresponding output in the
    response.
    :var productStrutureIdentifier: The structure ID  for the indexed product structure data. This identifier
    corresponds to the product id indexed in the VDS (and passed in the .mmp file).
    :var productAndConfig: The configuration that needs to be applied on the  product structure.
    :var revRuleInfo: The list of RevRuleInfo containing revision rule information to be used for solver library inside
    VDS.
    """
    clientId: str = ''
    productStrutureIdentifier: str = ''
    productAndConfig: ProductAndConfigInfo = None
    revRuleInfo: List[RevRuleInfo] = ()


@dataclass
class ProductStructureIdInput(TcBaseObj):
    """
    This structure contains product and the associated configuration for which the product structure information id is
    to be determined. The configuration information can either be passed in a detailed format or as recipe object.
    
    :var clientId: A clientId passed by the client in order to map the input to the corresponding output in the
    response.
    :var recipeObject: The object that contains the configuration recipe. The supported types are
    Awb0ProductContextInfo or a VisStructureContext object.
    :var productAndConfig: The product and configuration information for which the structure id has to be determined.
    """
    clientId: str = ''
    recipeObject: BusinessObject = None
    productAndConfig: ProductAndConfigInfo = None


@dataclass
class RevRuleInfo(TcBaseObj):
    """
    This structure contains revision rule entry names, effectivity expression, variant expression and revision rule
    date.
    
    :var revRuleEntryNames: A list of revision rule entry names. The revision rule entry clause names will be used by
    VDS for configuration solve.
    :var effectivityExpr: Effectivity boolean expression string. For eg. "Unit  >= 1 &amp;&amp; Unit < 100". This
    expression will be used by VDS for effectivity solve and is an internal syntax. The client does not parse the
    string. It should instead pass this to VDS to filer the node using this effectivity expression..
    :var variantExpr: Variant  boolean expression string. For eg. "Color = Red &amp;&amp;  Fuel=Petrol". This
    expression will be used by VDS for variant solve  and is an internal syntax. The client does not parse the string.
    It should instead pass this to VDS to filer the node using this variant expression.
    :var revRuleDate: Revision rule Date.
    :var context: String value which differentiates whether it&rsquo;s client or partition revision rule. Valid values
    are: "ClientRevRule" or "PartitionRevRule".
    """
    revRuleEntryNames: List[str] = ()
    effectivityExpr: str = ''
    variantExpr: str = ''
    revRuleDate: datetime = None
    context: str = ''


@dataclass
class VariantRuleInfo(TcBaseObj):
    """
    This structure contains  VariantRule objects that were applied for configuration and the object to which the
    VariantRule is associated through GRM relation.
    
    :var variantRule: VariantRule object applied for configuration.
    :var variantRuleOwningObject: The object to which the VariantRule is associated through ImanRelation . The
    variantRuleOwningObject is the object chosen when the user is selecting a configuration that needs to be applied.
    The user could use any relation filter while specifying it but it is typically specification, reference or
    manifestation relation.
    """
    variantRule: VariantRule = None
    variantRuleOwningObject: BusinessObject = None


@dataclass
class EffectivityInfo(TcBaseObj):
    """
    This structure contains  information about the effectivity configuration.
    
    :var effectivityUnitNo: Effective unit number. Use value of  -1 if unit effectivity is not to be considered for
    configuration.
    :var effectivityDate: Effective date configuration information.
    :var endItem: Effective  end item. Supported object types are:  ItemRevision
    """
    effectivityUnitNo: int = 0
    effectivityDate: datetime = None
    endItem: BusinessObject = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Awb0ProductContextInfo
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class OccurrenceFilterInput(TcBaseObj):
    """
    'OccurrenceFilterInput' specifies the information about recipe to search and filter the result.
    
    :var recipe: Recipe to provide search criteria.
    :var searchFilterCategories: A list of search filter categories.
    :var searchFilterMap: A map (string,list of SearchFilter)  containing the list of search filters for each search
    filter field.
    :var searchSortCriteria: A list of criterion to use to sort the filtered results.
    :var searchFilterFieldSortType: Type of sorting that needs to be performed on the search results. The fields can be
    sorted alphabetically in ascending order or in priority order from highest to lowest. The valid values are
    "Alphabetical" &amp; "Priority".
    :var fetchUpdatedFilters: A boolean flag to control filter data population in response. If true, response contains
    updated filters. If false, response does not contain any filter data. E.g. in case of scrolling down the contents,
    this flag can be set as false not to return updated filter as there won't be any updates.
    """
    recipe: List[Criteria] = ()
    searchFilterCategories: List[SearchFilterField2] = ()
    searchFilterMap: SearchFilterMap2 = None
    searchSortCriteria: List[SearchSortCriteria2] = ()
    searchFilterFieldSortType: str = ''
    fetchUpdatedFilters: bool = False


@dataclass
class OccurrenceFilterOutput(TcBaseObj):
    """
    OccurrenceFilterOutput specifies the output search criteria and updated filters.
    
    :var recipe: Updated recipe including the user specified filters that were provided in OccurrenceFilterInput.
    :var searchFilterCategories: A list of search filter categories.
    :var searchFilterMap: A map ( String, list of SearchFilter2 ) of search filter categories ( String ) with their
    corrosponding list of search filters ( SearchFilter2 ). Valid keys are' Ptn0PartitionScheme' and 'Design Component'.
    :var defaultCategoryDisplayCount: Number of categories display count.
    """
    recipe: List[Criteria] = ()
    searchFilterCategories: List[SearchFilterField2] = ()
    searchFilterMap: SearchFilterMap2 = None
    defaultCategoryDisplayCount: int = 0


@dataclass
class SearchFilter2(TcBaseObj):
    """
    'SearchFilter2' specifies the details of a filter. It indicates if it is a string, date or numeric type of search
    filter that is to be applied to the occurrences of the requested product.
    
    :var searchFilterType: The type of search filter to apply. Valid values are "StringFilter", "DateFilter",
    "NumericFilter" and  "HeirarchialFilter".
    :var stringValue: This specifies the internal text string value to filter by.
    :var startEndRange: The 'gap' used to generate the start and end values
    :var stringDisplayValue: This specifies the display value of the filter.
    :var startDateValue: The starting value for a date filter. This field is applicable only if the 'searchFilterType'
    field is set to "DateFilter".
    :var endDateValue: The ending value for a date filter. This field is applicable only if the 'searchFilterType'
    field is set to "DateFilter".
    :var startNumericValue: The starting value for a numeric filter. This field is applicable only if the
    'searchFilterType' field is set to "NumericFilter".
    :var endNumericValue: The ending value for a numeric filter. This field is applicable only if the
    'searchFilterType' field is set to "NumericFilter".
    :var count: The number of values in the filter. This field is populated on the service response and is ignored on
    the service input.
    :var selected: A flag that indicates if the filter was previously selected and used to filter the search results.
    This field is populated on the service response and is ignored on the service input.
    :var hasChildren: A flag indicating if the filter has children.
    """
    searchFilterType: str = ''
    stringValue: str = ''
    startEndRange: str = ''
    stringDisplayValue: str = ''
    startDateValue: datetime = None
    endDateValue: datetime = None
    startNumericValue: float = 0.0
    endNumericValue: float = 0.0
    count: int = 0
    selected: bool = False
    hasChildren: bool = False


@dataclass
class SearchFilterField2(TcBaseObj):
    """
    'SearchFilterField2' contains the resulting configured products found, that contain the queried object. This data
    structure also returns a cursor which can be sent back to a call to in order to rerieve more pages of results.
    
    :var internalName: The internal name for the search filter field.
    :var displayName: The display name for the search filter field.
    :var defaultFilterValueDisplayCount: The default number of search filter values to display within the search filter
    field.
    :var editable: If true, UI renders an editable text box instead of filter values. E.g. when 'categoryType' is
    "Attribute", in UI instead of displaying filter value a text field should be rendered.
    :var quickSearchable: If true, UI rendes a quick search text box. E.g. when 'categoryType' is "Partition", in UI a
    quick search text box is rendered along with all filter values.
    :var categoryType: The type of the filter category. Valid values are "StringMatch",  "Partition", "Attribute" 
    ,"ClosureRule", "SelectedElement"," BoxZone", "Proximity"," PlaneZone"
    :var isHierarchical: A boolean to indicate if hiearchical display is required for the filter values that belong to
    this categroy. If true, the filter values must have a hiearchical display in the client. Otherwise the filter
    values are shown as a flat list. Default value of this parameter is false.
    :var isMultiSelect: If true, multiple filters can be selected within this category. Otherwise only single filter
    can be selected. Default value of this parameter is true.
    """
    internalName: str = ''
    displayName: str = ''
    defaultFilterValueDisplayCount: int = 0
    editable: bool = False
    quickSearchable: bool = False
    categoryType: str = ''
    isHierarchical: bool = False
    isMultiSelect: bool = False


@dataclass
class SearchSortCriteria2(TcBaseObj):
    """
    'SearchSortCriteria2' specifies the criteria to use to sort the results that are retrieved. It provides the field
    to sort and the direction to sort it in.
    
    :var fieldName: The name of the field to perform the sorting on. This has to be the name of a property of an
    Awb0Element or its subtype on which to perform the sorting.
    :var sortDirection: The direction in which the sorting needs to be perfomed. It could be ascending or descending.
    Valid values are "ASC" and "DESC".
    """
    fieldName: str = ''
    sortDirection: str = ''


@dataclass
class SubsetResponse(TcBaseObj):
    """
    SubsetResponse contains a output subset recipe and filters for every given product context input.
    
    :var filterOut: A list of filter comprising the recipes and filters for the given Awb0ProducContextInfo.
    :var serviceData: Contains the list of all BusinessObject(s) that make up the output, as well as any errors that
    might have ocurred as part of the service invocation.
    """
    filterOut: List[OccurrenceFilterOutput] = ()
    serviceData: ServiceData = None


@dataclass
class Criteria(TcBaseObj):
    """
    'Criteria' provides information about recipe to search occurrences in a product.
    
    :var criteriaType: The type of search criteria represented in this structure. The list of valid values are: 
    - Attribute: Indicates a property based search term
    - BoxZone: Indicates a type of spatial search term
    - PlaneZone: Indicates a type of spatial search term
    - Proximity: Indicates a type spatial search term
    - ClosureRule: Indicates a closure rule search term
    - Group: Indicates a groups of multiple search terms
    - Partition: Indicates a partition based search term
    
    
    :var criteriaValues: A list of internal values used within this search term.
    :var criteriaDisplayValue: A user-friendly string representation of this search term.
    :var criteriaOperatorType: The search term operator indicating the logic for combining this search term. The list
    of valid values are:
    - Include (OR)
    - Filter (AND)
    - Exclude (NOT)
    
    
    :var subCriteria: A grouping of multiple search terms. This will be empty if the recipe contains no groups.
    """
    criteriaType: str = ''
    criteriaValues: List[str] = ()
    criteriaDisplayValue: str = ''
    criteriaOperatorType: str = ''
    subCriteria: List[Criteria] = ()


@dataclass
class EffectivityRange(TcBaseObj):
    """
    Structure to hold a row of effectivity data.
    
    :var unitIn: Unit at which this validity range starts. Valid value is non zero positive integer number.
    :var unitOut: Unit at which this validity range ends. Valid value is non zero positive integer number. Maximum
    value can be 2147483646 ( SO ) or 2147483647 ( UP ).
    :var dateIn: Date at which this validity range starts.
    :var dateOut: Date at which this validity range ends. Maximum value can be 26th Dec 9999 00:00:00 ( SO ) or 30th
    Dec 9999 00:00:00 ( UP ) in UTC timezone.
    """
    unitIn: int = 0
    unitOut: int = 0
    dateIn: datetime = None
    dateOut: datetime = None


@dataclass
class FilterSearchCriteria(TcBaseObj):
    """
    'FilterSearchCriteria' provides information related to the value entered in search filter field, as well as the
    category related details.
    
    :var categoryType: The category type under which search is requested. Valid values are:" Partition", "Attribute" ,
    "ClosureRule", "SelectedElement", "BoxZone", "Proximity", "PlaneZone"," StringMatch".
    :var categoryInternalName: The name of the category.
    :var searchString: User entered search text for finding the filters.
    """
    categoryType: str = ''
    categoryInternalName: str = ''
    searchString: str = ''


@dataclass
class FindMatchingFilterInput(TcBaseObj):
    """
    Input for 'findMatchingFilters'.
    
    :var productInfo: product context info.
    :var appliedCriteria: List of criteria which are already applied. This list is used to set selected parameter in
    returned SearchFilter2.
    :var searchCriteria: search criteria
    """
    productInfo: Awb0ProductContextInfo = None
    appliedCriteria: List[Criteria] = ()
    searchCriteria: FilterSearchCriteria = None


@dataclass
class FindMatchingFilterOutput(TcBaseObj):
    """
    'FindFilterOutput' provides output information related to the value entered in search filter field, as well as the
    category related details.
    
    :var searchFilterMap: Contains the map  (Filter category namestring, list of 'SearchFilter2' ) of category name to
    its display values.
    """
    searchFilterMap: SearchFilterMap2 = None


@dataclass
class FindMatchingFilterResponse(TcBaseObj):
    """
    'FindFilterResponse' contains output categories and their display values organized based on the search sort
    criteria specified in the input.
    
    :var filterOut: A list of filter output comprising the list of categories and assocaited filter values  for the
    given Awb0ProducContextInfo.
    :var serviceData: Contains the list of all BusinessObjects that make up the output, as well as any errors that
    might have ocurred as part of the service invocation.
    """
    filterOut: List[FindMatchingFilterOutput] = ()
    serviceData: ServiceData = None


"""
'SearchFilterMap2' is a map  (Filter category name, string  list of 'SearchFilter2' ) containing the list of search filters for each search filter field based on the search results.
"""
SearchFilterMap2 = Dict[str, List[SearchFilter2]]



from __future__ import annotations

from tcsoa.gen.AuthorizedDataAccess._2009_10.LicenseManagement import GetLicenseDetailsResponse, LicenseDetails, LicenseInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class LicenseManagementService(TcService):

    @classmethod
    def setLicenseDetails(cls, licenseInfo: List[LicenseDetails]) -> ServiceData:
        """
        This operation creates or modifies an ADA_License business object for each 'LicenseDetails' supplied. The
        'LicenseDetails' contains information for properties such as license type, license ID, expiry date, lock date,
        reason, in accordance with, and associated users and groups, for a given license. If a specified license ID
        already exists, the rest of the property values are updated on that license. However, if the license ID does
        not exist, then a new license of the specified type and ID will be created, and the rest of the properties are
        set on the created license. The user performing the operation will need the privilege specified in the
        ADA_license_administration_privilege site preference to create/modify an ADA License. If a user does not have
        the necessary privilege or if there is a validation error, the operation would fail and the error is returned
        in the 'ServiceData'.
        
        Use cases:
        Use Case 1: Create an ADA license
        You can create a new ADA license of type ITAR/IP/Exclude using 'setLicenseDetails' operation by providing a
        unique license ID for the license using 'LicenseDetails' structure.
        
        Use Case 2: Modify an ADA License
        The following types of modifications can be done on existing ADA licenses using 'setLicenseDetails' operation.
        Other than the license ID and license type, all other parameters of the ADA licenses can be modified using this
        operation.
        - Set an expiry or lock date on a license
        - Unlock licenses by specifying a NULL date or a future date for lockDate parameter
        - Add new users and/or groups to a license
        - Remove users and/or groups from a license
        - Specify value for qualifying code of federal regulations for ITAR licenses
        - Set reason for a license
        
        """
        return cls.execute_soa_method(
            method_name='setLicenseDetails',
            library='AuthorizedDataAccess',
            service_date='2009_10',
            service_name='LicenseManagement',
            params={'licenseInfo': licenseInfo},
            response_cls=ServiceData,
        )

    @classmethod
    def attachLicenses(cls, attachLicense: List[LicenseInput]) -> ServiceData:
        """
        This operation attaches ADA_License business objects to WorkspaceObject business objects such as Item,
        ItemRevision, Dataset, etc. as specified in each 'LicenseInput'. Optionally, this operation can set/update
        authorizing paragraph information for the ITAR licenses being attached. Users performing this operation will
        need IP_ADMIN privilege to both the workspace objects and the licenses being attached, if the licenses are of
        type IP or Exclude, while ITAR_ADMIN privilege is needed to both the workspace objects and the licenses being
        attached, if the licenses are of ITAR license type. If the user does not have necessary privilege to attach any
        licenses, or if there is any other error while attaching licenses, the errors are returned as partial errors in
        'ServiceData'.
        
        Use cases:
        Use case 1: Attach ADA licenses to WorkspaceObject business objects
        You can attach ADA_License business objects of ITAR/IP/Exclude type to classified WorkspaceObject business
        objects using 'attachLicenses' operation to grant access for users and/or groups listed on the licenses.
        Optionally, authorizing paragraph information can be specified for the ITAR licenses being attached through
        'LicenseInput' structure.
        
        Use Case 2: Modify authorizing paragraph for an attached ITAR license
        You can modify the authorizing paragraph information associated with an ITAR license already attached to a
        WorkspaceObject business object by passing in the license ID of the ITAR license, the WorkspaceObject business
        object and the modified authorizing paragraph information in 'LicenseInput' structure using 'attachLicenses'
        operation.
        """
        return cls.execute_soa_method(
            method_name='attachLicenses',
            library='AuthorizedDataAccess',
            service_date='2009_10',
            service_name='LicenseManagement',
            params={'attachLicense': attachLicense},
            response_cls=ServiceData,
        )

    @classmethod
    def getLicenseDetails2(cls, licenseIds: List[str]) -> GetLicenseDetailsResponse:
        """
        This operation gets the properties for each ADA_License business object specified in 'licenseIds' and returns
        them in 'LicenseDetails' structures as part of 'GetLicenseDetailsResponse'. The 'LicenseDetails' contains
        information for properties such as license type, license ID, expiry date, lock date, reason, in accordance
        with, and associated users and groups, for a given license. If there is no matching license for a given license
        ID, LicenseDetails structure for that license will contain 'Not_found' as the value for license ID, while the
        rest of the parameters will contain NULL values. However, if the user does not have READ privilege to any
        license or if there is any unexpected error while getting property information, the errors are returned in the
        'ServiceData' of 'GetLicenseDetailsResponse'.
        
        Use cases:
        Use Case 1: Get properties of an ADA License
        You can get properties of an ADA_License business object of type ITAR/IP/Exclude using 'getLicenseDetails2'
        operation by providing license ID for the license.
        
        Use Case 2: Check the existence of an ADA License
        You can check if an ADA_License businesses object with given license ID exists using 'getLicenseDetails2'
        operation.
        """
        return cls.execute_soa_method(
            method_name='getLicenseDetails2',
            library='AuthorizedDataAccess',
            service_date='2009_10',
            service_name='LicenseManagement',
            params={'licenseIds': licenseIds},
            response_cls=GetLicenseDetailsResponse,
        )



from tcsoa.gen.Internal_Integration._2007_06.services import IntegrationManagementService as imp0
from tcsoa.gen.Internal_Integration._2008_06.services import IntegrationManagementService as imp1
from tcsoa.utils import TcService


class IntegrationManagementService(TcService):
    connect = imp0.connect
    renameIMF = imp1.renameIMF



from tcsoa.gen.Requirementsmanagement._2007_01.services import RequirementsManagementService as imp0
from tcsoa.gen.Requirementsmanagement._2012_09.services import RequirementsManagementService as imp1
from tcsoa.gen.Requirementsmanagement._2008_06.services import RequirementsManagementService as imp2
from tcsoa.gen.Requirementsmanagement._2010_09.services import RequirementsManagementService as imp3
from tcsoa.gen.Requirementsmanagement._2009_10.services import RequirementsManagementService as imp4
from tcsoa.gen.Requirementsmanagement._2011_06.services import RequirementsManagementService as imp5
from tcsoa.utils import TcService


class RequirementsManagementService(TcService):
    createOrUpdate = imp0.createOrUpdate
    exportToApplication = imp0.exportToApplication
    getBomlineAfterCreate = imp1.getBomlineAfterCreate
    getRichContent = imp0.getRichContent
    getRichContent2 = imp2.getRichContent
    importFromApplication = imp0.importFromApplication
    moveLine = imp3.moveLine
    openStdNote = imp4.openStdNote
    publishColumnConfiguration = imp5.publishColumnConfiguration
    setRichContent = imp0.setRichContent
    setStdNote = imp4.setStdNote



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ItemRevision, BOMLine
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExpandOrUpdateDuplicateItemsInfo(TcBaseObj):
    """
    The 'ExpandOrUpdateDuplicateItemsInfo' contains the BOMLine objects, list of ItemRevision objects and list of
    Dependency types.
    
    :var bomline: If it is not null, expand it and get its dependencies based on the depTypes.  The BOMLine can be any
    line in the structure that the user picks for expansion.
    :var itemRevs: The ItemRevision objects to perform search dependencies on. If empty, the ItemRevision objects to be
    used will be from the expansion of the BOMLine.
    :var depTypes: The dependency types to use for the operation.
    """
    bomline: BOMLine = None
    itemRevs: List[ItemRevision] = ()
    depTypes: List[DependencyType] = ()


@dataclass
class ExpandOrUpdateDuplicateItemsOutput(TcBaseObj):
    """
    The 'ExpandOrUpdateDuplicateItemsOutput' represents the collection of all related data objects with the structure
    ItemRevision object(s) or part family masters ItemRevision object(s).
    
    :var object: The BOM structure ItemRevision.
    :var relatedObjInfo: List of all dependent objects that are related to the object.
    """
    object: ItemRevision = None
    relatedObjInfo: List[RelatedItemsInfo] = ()


@dataclass
class ExpandOrUpdateDuplicateItemsResponse(TcBaseObj):
    """
    The 'ExpandOrUpdateDuplicateItemsResponse' structure represents the the results of the structure dependent data
    search.
    
    :var outputs: List of 'ExpandOrUpdateDuplicateItemsOutput' which contains information about the BOM structure
    ItemRevision to its related data relationships.
    :var serviceData: The error information will be recorded here.
    """
    outputs: List[ExpandOrUpdateDuplicateItemsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class NewItemInfo(TcBaseObj):
    """
    The proposed new ItemIds and new ItemNames.
    
    :var newItemId: The proposed new ItemId.
    :var newItemName: The proposed new ItemName.
    """
    newItemId: str = ''
    newItemName: str = ''


@dataclass
class RelatedItemsInfo(TcBaseObj):
    """
    The RelatedItemsInfo represents the dependent data found and how it is related to a structure ItemRevision.
    RelatedItemsInfo also can represent a family table member to a family table master.
    
    :var relObj: An ItemRevision  - it is the related data.
    :var relationshipName: The GRM relationship used to find the related data.
    :var depType: The cad dependency types to use for the operation.
    """
    relObj: ItemRevision = None
    relationshipName: str = ''
    depType: DependencyType = None


@dataclass
class ValidateStructureItemIdsInfo(TcBaseObj):
    """
    Input to ValidateStructureItemIds. It contains top BOM line, map of oldItemId to NewItemInfo(NewItemId,
    NewItemName), default naming pattern and list of dependency types specified by the user.
    
    :var topLine: Top BOMLine of the structure to be cloned.
    :var inputMap: Map of oldItemId and NewItemInfo structure.
    :var defaultName: The pattern to form new ItemIds.
    :var options: A list of dependency types.
    """
    topLine: BOMLine = None
    inputMap: DuplicateIdMap = None
    defaultName: DefaultNaming = None
    options: List[DependencyType] = ()


@dataclass
class ValidateStructureItemIdsResponse(TcBaseObj):
    """
    Response from 'ValidateStructureItemIds', containing map of OldItemID to 'NewItemInfo', list of ItemRevision
    objects that failed validation and the 'ServiceData' object.
    
    :var clonedIdMap: A map of OldItemID to NewItemInfo that contains the proposed New ItemId and New ItemName.
    :var serviceData: 'ServiceData' object.  The error stack will contain those ItemRevision objects that failed
    cloning and the reason.
    """
    clonedIdMap: DuplicateIdMap = None
    serviceData: ServiceData = None


@dataclass
class DefaultNaming(TcBaseObj):
    """
    prefix/suffix/replace-with/autoAssign to form new Item IDs.
    
    :var autogen: auto assign new item IDs.
    :var prefix: substring to be prefixed to the old item ID to form a new item ID.
    :var suffix: substring to be sufficed to the old item ID to form a new item ID.
    :var from: substring to be replaced from the old item ID by another substring to form a new item ID.
    :var to: substring to replace with a substring from the old item ID to form a new item ID.
    """
    autogen: bool = False
    prefix: str = ''
    suffix: str = ''
    from_: str = ''
    to: str = ''


@dataclass
class DuplicateInputInfo(TcBaseObj):
    """
    Input for Duplicate SOA API
    
    :var topLine: The top BOMLine of the structure to be cloned.
    :var clonedIdMap: A map of oldItemId and NewItemInfo structure.  The newItemInfo is made up of the proposed new
    ItemId and the proposed new ItemName.
    :var defaultName: A pattern to form a new ItemId.
    :var renameCadFile: Whether to rename the CAD files or not.
    :var options: A list of dependency types. This defines the cad traversal.
    """
    topLine: BOMLine = None
    clonedIdMap: DuplicateIdMap = None
    defaultName: DefaultNaming = None
    renameCadFile: bool = False
    options: List[DependencyType] = ()


@dataclass
class DuplicateResponse(TcBaseObj):
    """
    response from the Duplicate SOA.
    
    :var clonedItemRev: Top ItemRevision of the Cloned structure.
    :var serviceData: 'ServiceData' object - The service data object consists of the   top ItemRevision of the cloned
    structure.  The error stack will contain those ItemRevision objects that failed cloning and the reason.  Those
    ItemRevision objects that fail to get cloned are referenced.
    """
    clonedItemRev: ItemRevision = None
    serviceData: ServiceData = None


class DependencyType(Enum):
    """
    Dependency Types
    """
    NoDep = 'NoDep'
    Drawing = 'Drawing'
    PartFamilyMaster = 'PartFamilyMaster'
    PartFamilyMember = 'PartFamilyMember'
    Required = 'Required'
    AllDep = 'AllDep'
    ExcludeFromBom = 'ExcludeFromBom'


"""
OldItemIds mapped to the NewItemInfo structure.
"""
DuplicateIdMap = Dict[str, NewItemInfo]



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetOfficeStyleSheetResponse(TcBaseObj):
    """
    The list of Office template stylesheets attached to the ReportDefinition.
    
    :var officetemplates: The names of the office template stylesheets attached to the ReportDefinition object via the
    GRM relationship.
    :var serviceData: The standard ServiceData
    """
    officetemplates: List[str] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindObjectsAttrAndValues(TcBaseObj):
    """
    A structure for FindObjectsByClassAndAttributes operation.  This structure contains the  attributes and values used
    to filter the search.
    
    :var attrName: Attribute name
    :var values: List of attribute values
    """
    attrName: str = ''
    values: List[str] = ()


@dataclass
class FindObjectsInput(TcBaseObj):
    """
    Input stucture for FindObjectsByClassAndAttributes.
    
    :var className: An attribute name on the class to query on.
    :var clientId: The client ID
    :var outputAttributeNames: List of attributes whos values will be returned.  Can be empty if requesting the objects.
    :var attrAndValues: A list of FindSingleAttributeAttrAndValues.
    """
    className: str = ''
    clientId: str = ''
    outputAttributeNames: List[str] = ()
    attrAndValues: List[FindObjectsAttrAndValues] = ()


@dataclass
class FindObjectsResponse(TcBaseObj):
    """
    Return structure for FindObjectsByClassAndAttributes operation.
    
    :var serviceData: The ServiceData
    :var result: A list of the objects found.  This will be empty if attribute values are requested.
    :var attrAndValues: A list of FindSingleAttributeAttrAndValues.
    """
    serviceData: ServiceData = None
    result: List[BusinessObject] = ()
    attrAndValues: List[FindObjectsAttrAndValues] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ItemRevision, BOMLine
from typing import List
from tcsoa.gen.Internal.Cae._2014_06.StructureManagement import MonitorResponse
from tcsoa.utils import TcService


class StructureManagementService(TcService):

    @classmethod
    def loadSimulationDataMonitor(cls, rootbomline: BOMLine) -> MonitorResponse:
        """
        CAE Manager provides capabilities to create and manage CAE data. Often this consists of a simulation structure
        where each component is expected to have certain attribute values set and manage certain types of data. The
        analyst wants to quickly assess pertinent attribute values and existence of pertinent data of all components of
        a simulation structure. 
        The CAE Manager application will now provides the capability to configure and launch a "Simulation Data
        Monitor" to access the status of components loaded in the model view.
        This operation will get the simulation data monitor configuration from the client side (RAC) and depending upon
        the configuration it will get the status of components loaded in the model view.
        
        Use cases:
        This operation get the valid simulation data monitor configuration and when open simulation data monitor view
        command is invoked on the loaded simulation strucure on Model view, operation prepare a response containing the
        status of monitored components    attributes and attached files as per the simulation data monitor
        configuration.
        
        Step:
        1. Opens CAE Manager Application.
        2. Load simulation structure on Model view.
        3. Clicks on "Open Secondary Views" drop down.
        4. Select "Simulation Data Monitor" view from drop down list.
        """
        return cls.execute_soa_method(
            method_name='loadSimulationDataMonitor',
            library='Internal-Cae',
            service_date='2014_06',
            service_name='StructureManagement',
            params={'rootbomline': rootbomline},
            response_cls=MonitorResponse,
        )

    @classmethod
    def refreshSimulationDataMonitor(cls, itemrevisions: List[ItemRevision]) -> MonitorResponse:
        """
        CAE Manager provides capabilities to create and manage CAE data. Often this consists of a simulation structure
        where each component is expected to have certain attribute values set and manage certain types of data. The
        analyst wants to quickly assess pertinent attribute values and existence of pertinent data of all components of
        a simulation structure. 
        The CAE Manager application will now provides the capability to configure and launch a "Simulation Data
        Monitor" to access the status of components loaded in the model view.
        The analyst can leave the Simulation Data Monitor active for any desired length of time.
        CAE Manager will provide a refresh command in the monitor which will refresh the status of selected components.
        This operation will use the simulation data monitor configuration used at the time of load  and apply it again
        for the list of selected compoents from model view.
        
        
        Use cases:
        This operation get the simulation data monitor configuration used at the time load operation and when open
        refresh command is invoked on the selected components of simulation data monitor detail table, operation
        prepare a response containing the status of selected components attributes and attached files as per the
        simulation data monitor configuration.
        
        Step:
        1. Opens CAE Manager Application.
        2. Load simulation structure on Model view.
        3. Actor opens "Simulation Data Monitor" view from Model view.
        4. Select component(s) appearing on detail table.
        5. Clicks refresh icon appear on toolbar of Simulation Data Monitor view.
        """
        return cls.execute_soa_method(
            method_name='refreshSimulationDataMonitor',
            library='Internal-Cae',
            service_date='2014_06',
            service_name='StructureManagement',
            params={'itemrevisions': itemrevisions},
            response_cls=MonitorResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.GlobalMultiSite._2007_06.ImportExport import DryRunExportResponse, TransferFormula, ImportObjectsResponse, OwningSiteAndObjs, CallToRemoteSiteResponse, ConfirmExportResponse, NamesAndValues, ExportObjectsResponse
from tcsoa.gen.Internal.GlobalMultiSite._2007_06.Synchronizer import ReplicaDeletionMasterUpdateResponse, ObjectsByClass, OwnershipChangeReplicaUpdateResponse, SyncResponse, CheckSyncStateResponse, StubReplicationMasterUpdateResponse
from typing import List
from tcsoa.gen.Internal.GlobalMultiSite._2007_06.Briefcase import PackageBriefcaseContentsResponse, UnpackBriefcaseContentsResponse, PackageBriefcaseContentsInfo
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class SynchronizerService(TcService):

    @classmethod
    def checkReplicaSyncState(cls, replicaSite: int, syncType: str, revisionRule: str, inputReplicaGSIdentityList: List[str]) -> CheckSyncStateResponse:
        """
        For all objects, pointed by input GSIdentity, returns up to date or out of date sync state based on object
        modification since the last export. This operation supports only object level synchronization.
        
        Use cases:
        From RAC, when the user selects &#39;Tools&#45;>MultiSite collaboration&#45;> Check replica sync state&#39;,
        this operation gets invoked and status gets displayed. This operation also gets called from TC Enterprise.
        """
        return cls.execute_soa_method(
            method_name='checkReplicaSyncState',
            library='Internal-GlobalMultiSite',
            service_date='2007_06',
            service_name='Synchronizer',
            params={'replicaSite': replicaSite, 'syncType': syncType, 'revisionRule': revisionRule, 'inputReplicaGSIdentityList': inputReplicaGSIdentityList},
            response_cls=CheckSyncStateResponse,
        )

    @classmethod
    def updateMasterObjectsOnReplicaDeletion(cls, replicaSite: int, inputReplicaGSIdentityList: List[str], convertToStub: bool) -> ReplicaDeletionMasterUpdateResponse:
        """
        This is an internal operation which updates ImanExportRecord of deleted object. If replica object(s) are
        converted to stub then export status of ImanExportRecord gets updated to &#39;Covert to stub&#39;. If replica
        gets deleted, export status is updated to &#39;Deleted&#39;.
        
        Use cases:
        This operation gets triggered when replica workspace object gets deleted. For each workspace object, SOAP call
        is made to Global Services which in turn calls updateMasterObjectsOnReplicaDeletion on owning site to update
        ImanExpordRecord status.
        """
        return cls.execute_soa_method(
            method_name='updateMasterObjectsOnReplicaDeletion',
            library='Internal-GlobalMultiSite',
            service_date='2007_06',
            service_name='Synchronizer',
            params={'replicaSite': replicaSite, 'inputReplicaGSIdentityList': inputReplicaGSIdentityList, 'convertToStub': convertToStub},
            response_cls=ReplicaDeletionMasterUpdateResponse,
        )

    @classmethod
    def updateObjectsOnOwnershipChange(cls, newOwningSite: int, inputReplicaGSIdentityList: List[str]) -> OwnershipChangeReplicaUpdateResponse:
        """
        This operation updates the owning site of replica objects pointed by input GSIdentities in the context of
        ownership change.
        
        Use cases:
        This operation is called by Global Services BPEL process in the context of ownership change. The owning site of
        objects which are in input list is changed to newOwningSite.
        """
        return cls.execute_soa_method(
            method_name='updateObjectsOnOwnershipChange',
            library='Internal-GlobalMultiSite',
            service_date='2007_06',
            service_name='Synchronizer',
            params={'newOwningSite': newOwningSite, 'inputReplicaGSIdentityList': inputReplicaGSIdentityList},
            response_cls=OwnershipChangeReplicaUpdateResponse,
        )

    @classmethod
    def createExportRecordOnStubReplication(cls, replicaSite: int, transferFormula: str, transactionID: str, inputReplicaGSIdentityList: List[str]) -> StubReplicationMasterUpdateResponse:
        """
        Creates export records for all objects pointed by input GSIdentities on the master site in response to the stub
        object creation on replica site. This is an internal operation that is not exposed to the customers.
        
        Use cases:
        This operation is not in use.
        """
        return cls.execute_soa_method(
            method_name='createExportRecordOnStubReplication',
            library='Internal-GlobalMultiSite',
            service_date='2007_06',
            service_name='Synchronizer',
            params={'replicaSite': replicaSite, 'transferFormula': transferFormula, 'transactionID': transactionID, 'inputReplicaGSIdentityList': inputReplicaGSIdentityList},
            response_cls=StubReplicationMasterUpdateResponse,
        )

    @classmethod
    def getCandidatesToSynchronizeForListOfClasses(cls, targetSite: int, classList: List[str]) -> SyncResponse:
        """
        This is an internal operation which returns a list of candidates that need to be synchronized based on input
        class list and target site.
        
        Use cases:
        
        """
        return cls.execute_soa_method(
            method_name='getCandidatesToSynchronizeForListOfClasses',
            library='Internal-GlobalMultiSite',
            service_date='2007_06',
            service_name='Synchronizer',
            params={'targetSite': targetSite, 'classList': classList},
            response_cls=SyncResponse,
        )

    @classmethod
    def getCandidatesToSynchronizeForListOfObjects(cls, targetSite: int, objectsListToProcess: List[ObjectsByClass]) -> SyncResponse:
        """
        This is an internal operation which returns a list of candidates that need to be synchronized based on the
        input set of classes (mandatory) or list of objects (optional). Objects that are replicated at target site are
        taken into account.
        
        Use cases:
        
        """
        return cls.execute_soa_method(
            method_name='getCandidatesToSynchronizeForListOfObjects',
            library='Internal-GlobalMultiSite',
            service_date='2007_06',
            service_name='Synchronizer',
            params={'targetSite': targetSite, 'objectsListToProcess': objectsListToProcess},
            response_cls=SyncResponse,
        )


class DataMigrationService(TcService):

    @classmethod
    def getRUO(cls) -> bool:
        """
        Returns the Replica Update Overide (RUO) value if the Global MultiSite (GMS) is in replicabypass mode,
        otherwise a false is returned.
        """
        return cls.execute_soa_method(
            method_name='getRUO',
            library='Internal-GlobalMultiSite',
            service_date='2007_06',
            service_name='DataMigration',
            params={},
            response_cls=bool,
        )

    @classmethod
    def setRUO(cls, key: str, value: bool) -> ServiceData:
        """
        This operation sets a given logical 'value', if the GMS is in replicabypass mode.Gives an error as Mismatch key
        if GMS is not in replicabypass mode.
        """
        return cls.execute_soa_method(
            method_name='setRUO',
            library='Internal-GlobalMultiSite',
            service_date='2007_06',
            service_name='DataMigration',
            params={'key': key, 'value': value},
            response_cls=ServiceData,
        )


class ImportExportService(TcService):

    @classmethod
    def importObjects(cls, masterSite: int, tcGSMessageId: str, fmsTickets: List[str], inputData: TransferFormula, synchronize: bool) -> ImportObjectsResponse:
        """
        Imports a list of objects and its dependents in Teamcenter XML (Tc XML)format from a specified source site
        based on the input transaction id, list of FMS tickets of the input TcXML files to be imported,
        'TransferFormula' and type of transfer(initial or synchronize).After the import, the objects would be owned by
        target site if ownership transfer session option is specified or they would be imported as replica objects
        meaning the ownership of the objects imported lies with the source site.
        
        Use cases:
        This operation is used in conjunction with exportObjects operation. importOperation will be executed at each
        target site after the exportObjects is called by the BPEL engine during both high level(Not so fast) and low
        level (Fast) data transfer. The low level use case is nothing but site consolidation triggered by teamcenter
        utility 'sitcons_replicate_mg'r which replicates the data from a source site to the target site using the TcXML
        low level (Fast transfer) transfer options. To perform fast transfer the session option name value pair has to
        be 
        - name=fastStream
        - value=yes
        
        
        If the session option fastStream is not specified it will be a high level transfer. Both high level and low
        level transfer require Global Services configured between two sites. TcXML format and schema are different in
        case of high level and low level transfer. The easiest way to identify a TcXML is to look for attribute
        format=high_level in case of high level and format=low_level  in case of low_level.
        """
        return cls.execute_soa_method(
            method_name='importObjects',
            library='Internal-GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'masterSite': masterSite, 'tcGSMessageId': tcGSMessageId, 'fmsTickets': fmsTickets, 'inputData': inputData, 'synchronize': synchronize},
            response_cls=ImportObjectsResponse,
        )

    @classmethod
    def confirmExport(cls, targetSite: int, tcGSMessageId: str, fmsTicketOfFailedObjs: str, commit: bool) -> ConfirmExportResponse:
        """
        This operation should be called at the source site during data transfer after the import is done at the target
        site. First export at source site, import at target site and then a confirm export of failed/successful objects
        at source site. When export is done at the source site export records are created. 'confirmExport' will then
        update the export status in Export record of objects back at the source site after import based on the target
        site id, transaction id and the File Management System (FMS) tickets of file with failed objects.The caller is
        responsible to use FMS to transfer data files from one site to another.
        
        Use cases:
        This operation marks the data transfer from source site to target site as complete by reporting the status of
        objects imported at target site back to source site.
        """
        return cls.execute_soa_method(
            method_name='confirmExport',
            library='Internal-GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'targetSite': targetSite, 'tcGSMessageId': tcGSMessageId, 'fmsTicketOfFailedObjs': fmsTicketOfFailedObjs, 'commit': commit},
            response_cls=ConfirmExportResponse,
        )

    @classmethod
    def requestExportToRemoteSites(cls, reason: str, sites: List[BusinessObject], objects: List[BusinessObject], optionSet: BusinessObject, optionNameAndValues: List[NamesAndValues], sessionOptionNamesAndValues: List[NamesAndValues]) -> CallToRemoteSiteResponse:
        """
        This operation exports objects to specified sites using Global Multisite.
        
        Use cases:
        This operation gets invoked from RAC when user does following actions from Navigator or structure manager
        1>    Tools->Export->To Remote Site Via Global Services.
        2>    Tools->Export->To PDX
        3>    Tools->Export->To Briefcase
        """
        return cls.execute_soa_method(
            method_name='requestExportToRemoteSites',
            library='Internal-GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'reason': reason, 'sites': sites, 'objects': objects, 'optionSet': optionSet, 'optionNameAndValues': optionNameAndValues, 'sessionOptionNamesAndValues': sessionOptionNamesAndValues},
            response_cls=CallToRemoteSiteResponse,
        )

    @classmethod
    def requestImportFromRemoteSites(cls, reason: str, owningSitesAndObjs: List[OwningSiteAndObjs], optionSet: BusinessObject, optionNamesAndValues: List[NamesAndValues], sessionOptionNamesAndValues: List[NamesAndValues]) -> CallToRemoteSiteResponse:
        """
        Remote imports objcets to specified sites. E-mail notification would be sent to
        current user if session option for email notification is set to true.
        
        Use cases:
        This operation is not in use.
        """
        return cls.execute_soa_method(
            method_name='requestImportFromRemoteSites',
            library='Internal-GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'reason': reason, 'owningSitesAndObjs': owningSitesAndObjs, 'optionSet': optionSet, 'optionNamesAndValues': optionNamesAndValues, 'sessionOptionNamesAndValues': sessionOptionNamesAndValues},
            response_cls=CallToRemoteSiteResponse,
        )

    @classmethod
    def dryRunExport(cls, targetSites: List[int], expObjList: List[str], tcGSMessageId: str, inputData: TransferFormula) -> DryRunExportResponse:
        """
        Provides information about an export as if the real export was done and generates a report, based on the input
        'TransferFormula' and global services transaction id for a list of objects which have to be exported.
        
        Use cases:
        This will be executed at the source site and is invoked by Business Process Execution Language(BPEL ) engine
        component of Teamcenter GlobalServices.
        """
        return cls.execute_soa_method(
            method_name='dryRunExport',
            library='Internal-GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'targetSites': targetSites, 'expObjList': expObjList, 'tcGSMessageId': tcGSMessageId, 'inputData': inputData},
            response_cls=DryRunExportResponse,
        )

    @classmethod
    def exportObjects(cls, targetSites: List[int], expObjList: List[str], tcGSMessageId: str, inputData: TransferFormula, synchronize: bool) -> ExportObjectsResponse:
        """
        Exports a list of objects and its dependents in Teamcenter XML Tc XMLformat to specified target sites based on
        the input 'TransferFormula' and type of transfer initial or synchronize.The output TcXML file contains elements
        and its attributes which are traversed by applying the Transfer formula on items specified in input list.
        
        Use cases:
        Business Process Execution Language  BPEL  engine calls this operation to perform high level Not so fast and
        low level Fast transfer. This operation is used to do a data transfer from source site to target sites in case
        of high level. This is executed at the source site in both cases. For a low level transfer it is used by
        'Siteconsolidation' utility 'sitcons_replicate_mgr' which replicates the data from a source site to the target
        site using the TcXML low level Fast transfer options.This utility needs TcGS to be running and configured for
        site consolidation. To perform fast transfer the session option name value pair has to be 
        - name=fastStream 
        - value=yes
        
        
        If the session option fastStream is not specified it then it will be a high level transfer.In other words the
        input for both low level and high level are same except for options.
        """
        return cls.execute_soa_method(
            method_name='exportObjects',
            library='Internal-GlobalMultiSite',
            service_date='2007_06',
            service_name='ImportExport',
            params={'targetSites': targetSites, 'expObjList': expObjList, 'tcGSMessageId': tcGSMessageId, 'inputData': inputData, 'synchronize': synchronize},
            response_cls=ExportObjectsResponse,
        )


class BriefcaseService(TcService):

    @classmethod
    def packageBriefcaseContents(cls, packageBriefcaseContentsInfo: PackageBriefcaseContentsInfo) -> PackageBriefcaseContentsResponse:
        """
        This operation packages TC XML file and the associated named reference files to an offline package called
        Briefcase. The input TCXML files may reference Iman files which are FMS tickets in the XML files.
        The Iman files referenced in the TC XML files are downloaded to the transient volume, the source TC XML file is
        updated to convert the FMS tickets to relative paths. A briefcase file is created by packing the TC XML files
        and related files. The briefcase file is attached to a briefcase dataset, and is sent to the mail folder of the
        user. 
        This operation should be used together with ImportExport::exportObjects. The exportObjects operation will
        generate some TC XML files, and those files are the input to this operation.
        
        
        Use cases:
        User performs a briefcase export on some selected objects through Global Services. After TC XML export is
        completed, the server will have the FMS tickets to TC XML files and export log files. Then it will invoke this
        operation to pack the briefcase. When completed, user can check its mail folder and find the briefcase file in
        the attached dataset.
        
        Exceptions:
        >ServiceException    If this operation cannot complete the whole process. The error may be but not limit to, a
        failure to download a file due to FMS issue; failures to create the briefcase file; failures to create the
        dataset; failures to send the mail.
        """
        return cls.execute_soa_method(
            method_name='packageBriefcaseContents',
            library='Internal-GlobalMultiSite',
            service_date='2007_06',
            service_name='Briefcase',
            params={'packageBriefcaseContentsInfo': packageBriefcaseContentsInfo},
            response_cls=PackageBriefcaseContentsResponse,
        )

    @classmethod
    def unpackBriefcaseContents(cls, uidOfBriefcaseTcFile: str) -> UnpackBriefcaseContentsResponse:
        """
        Unpack a specific briefcase file. 
        This operation reads the briefcase file and extracts all the files from briefcase into transient volume;
        updates the TCXML file to convert the relative path of referenced files to FMS tickets; then return the FMS
        tickets of the updated TCXML files. 
        When unpack completed, the TCXML file is ready for import.
        This operation is invoked by the Global services.
        
        
        Use cases:
        User selects a briefcase file in TC server to do briefcase import. The server gets the UID of the briefcase
        file; unpack the briefcase file. After this operation completes, the server will get the TCXML files that are
        ready for TCXML import.
        
        Exceptions:
        >ServiceException    If this operation cannot complete the whole process. The error may be but not limit to, a
        failure to get the briefcase file from UID; failures to get file from ticket or failures to get ticket from
        file due to FMS issue; failures to unpack the briefcase file.
        """
        return cls.execute_soa_method(
            method_name='unpackBriefcaseContents',
            library='Internal-GlobalMultiSite',
            service_date='2007_06',
            service_name='Briefcase',
            params={'uidOfBriefcaseTcFile': uidOfBriefcaseTcFile},
            response_cls=UnpackBriefcaseContentsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindNodeInContextInputInfo(TcBaseObj):
    """
    Input struct for the find node in context service
    
    :var clientID: Client ID
    :var context: The topline that defines the search scope.
    :var nodes: The nodes to search.
    :var byIdOnly: If true all abs occs with the same Id will be search for, if no exact apn is matched.
    :var allContexts: If true then all contexts will be searched otherwise only the current context will be searched if
    no current context specified at the time then the context of the topline is used.
    :var inContextLine: A more specific scope to searh in.
    """
    clientID: str = ''
    context: BusinessObject = None
    nodes: List[BusinessObject] = ()
    byIdOnly: bool = False
    allContexts: bool = False
    inContextLine: BusinessObject = None


@dataclass
class FindNodeInContextResponse(TcBaseObj):
    """
    The Response struct.
    
    :var resultInfo: Infornation retrieves for each input struct that we looked for.
    :var serviceData: Service data.
    """
    resultInfo: List[FoundNodesInfo] = ()
    serviceData: ServiceData = None


@dataclass
class FoundNodesInfo(TcBaseObj):
    """
    A struct that hold all the results on a single search parallel to the one input struct.
    
    :var clientID: Client ID
    :var resultNodes: In each search we look into a vector of nodes to search.
    """
    clientID: str = ''
    resultNodes: List[NodeInfo] = ()


@dataclass
class GetAffectedPropertiesArg(TcBaseObj):
    """
    This structure provides the input parameters for the getAffectedProperties operation.  It describes the property
    changes applied to a process or operation structure for which the affected runtime properties are inquired.
    
    :var rootNode: he root node of the process or operation structure that is inspected.
    :var changedNodes: The list of process or operation nodes that have been changed.
    :var changedProperties: The name of the properties that have been changed.
    """
    rootNode: BusinessObject = None
    changedNodes: List[BusinessObject] = ()
    changedProperties: List[str] = ()


@dataclass
class NodeInfo(TcBaseObj):
    """
    The struct that holds the result for each node in the input.
    
    :var foundNodes: The found nodes we find.
    :var originalNode: The original input node.
    """
    foundNodes: List[BusinessObject] = ()
    originalNode: BusinessObject = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class SetContentInput(TcBaseObj):
    """
    DEPRECATED: SetContentInput structure represents the parameters required to set the contents to the FullText object.
    
    :var objectToProcess: SpecElement Revision or FullText object.
    :var bodyText: Value of the new body text on the FullText object.
    :var contentType: Value of the content type on the FullText object.
    """
    objectToProcess: WorkspaceObject = None
    bodyText: str = ''
    contentType: ContentTypes = None


@dataclass
class ExportToApplicationInputData(TcBaseObj):
    """
    Export to application input data
    
    :var objectsToExport: Objects to Export.
    :var attributesToExport: Attributes to export.
    :var applicationFormat: Application Format
    :var templateId: Template ID
    """
    objectsToExport: List[BusinessObject] = ()
    attributesToExport: List[str] = ()
    applicationFormat: str = ''
    templateId: str = ''


@dataclass
class ExportToApplicationResponse(TcBaseObj):
    """
    ExportToApplicationResponse
    
    :var transientFileReadTickets: transientFileReadTickets
    :var outlineMap: outline map
    :var serviceData: service data
    """
    transientFileReadTickets: List[str] = ()
    outlineMap: OutlineMap = None
    serviceData: ServiceData = None


class ContentTypes(Enum):
    """
    DEPRECATED: ContentTypes enumeration represents the valid content types of the FullText object.
    """
    HTML = 'HTML'
    PLAINTEXT = 'PLAINTEXT'


"""
Outline Map
"""
OutlineMap = Dict[str, BusinessObject]



from __future__ import annotations

from tcsoa.gen.Visualization._2011_02.StructureManagement import CreateVisSCsFromBOMsResponse, CreateVisSCInfo, CreateVisSCResponse, CreateVisSCsFromBOMsInfo
from tcsoa.gen.Visualization._2011_02.DataManagement import ServerInfo, VVITicketsResponse, IdInfo, SessionInfo, UserAgentDataInfo
from typing import List
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def createLaunchFile(cls, idInfos: List[IdInfo], serverInfo: ServerInfo, userDataAgentInfo: UserAgentDataInfo, sessionInfo: SessionInfo) -> VVITicketsResponse:
        """
        This service generates a VVI file which is used to launch Teamcenter Visualization viewers with selected
        objects from Teamcenter and preserve a two way communication link between the viewer and the server.  These
        files are not intended to be permanent and should be generated with each launch.  For example, the VVI format
        is not guaranteed to be supported if the server or viewer is updated.  VFZ files are used if more than one
        object is launched at a time.
        
        This operation requires the Teamcenter File Management System (FMS) to be installed (including FCC and
        transient volumes) for retrieval of the VVI file from the transient file volume.  This operation generates the
        launch file (VFZ or VVI), stores it in the FMS transient volume, and returns the FMS ticket.  The client that
        initiated this operation is responsible for downloading the transient file (VVI or VFZ) with the transient
        ticket from transient volume to a local file system. Transient (VVI or VFZ) file is then consumed by the
        Teamcenter Visualization client.  The viewer will then establish the server connection and load the object(s)
        specified in the VVI file.  Launch on multiple objects will generate a VFZ file (zip of all the vvi files) and
        transient ticket of VFZ file would be sent to client. 
        
        This service supports launch on Teamcenter persistent objects like Dataset, Item, ItemRevision,
        BOMViewRevision, BOMView. It also supports launch of selected BOMLines of a configured structure from Structure
        Manager or BOPLines from Manufacturing Process Planner, but in this case the caller must first create a
        VisStructureContext object (See Dependency section for operation to use).  Valid launch object types and
        behavior such as priority order can be configured with the Teamcenter Preferences VMU_Datasets,
        VMU_FileSearchOrder and VMU_RelationSearchOrder.
        
        
        Use cases:
        This operation supports the mechanism of visualizing Teamcenter specific objects in Teamcenter Visualization
        client. There are several steps to support this mechanism.
        
        1.    The client application that initiates the launch will provide:
        
        A vector of IdInfo objects that contains one or more pieces of information about Teamcenter objects that needs
        to be visualized in the viewer (e.g., If Dataset is launched, then information about its Item, ItemRevision and
        type of operation, including any additional information can be provided). Note: In case launch of Teamcenter
        runtime objects like BOMLines from Structure Manager or BOPLines from Manufacturing Process Planner, it is the
        responsibility of the client to create VisStructureContext object and provide VisStructureContext as the object
        to be laud (See Dependency section for operation to use).
        SessionInfo object contains session relevant information for Teamcenter Visualization to connect to the session
        (e.g., session discriminator and any other additional session relevant key value pair)
        ServerInfo object contains server information for Teamcenter Visualization to connect to the server. (e.g.,
        protocol, server URL, connection mode of the server and any other additional server relevant key value pair)
        UserAgentDataInfo object contains client application information who initiated the launch. (e.g., application
        name, application version, and any other additional client application relevant key value pair).
        
        
        2.    After gathering the necessary information as listed in step 1, client application then invokes the
        'DataManagementService::createLaunchFile' operation to obtain an FMS read ticket for the launch file (VVI or
        VFZ), that has relevant information for visualizing Teamcenter persistent or runtime objects.
        
        See the Dependencies section below for details.
        
        3.    Use a File Management System (FMS) Application Programmatic Interface (API) to download the transient
        file (VVI or VFZ) from transient volume.
        
        Check the documentation for each API to determine how to react to download failures.
        
        Use Case References:
        This operation is used in conjunction with other 'FileManagementService' service operations, 'Visualization'
        service operations, FccProxy and the FileManagementUtility. Please consult the documentation for each of these
        available operations for details on the requirements, usage, and environments in which they should be used. 
        
        Visualization operations for creating the VisStructureContext from clients that initiate the launch of
        Teamcenter runtime objects like BOMLines from Structure Manager or BOPLines from Manufacturing Process Planner.
        
        'Teamcenter::Soa::Internal::Visualization::_2011_02:: StructureManagement::CreateVisSCsFromBOMsResponse
        createVisSCsFromBOMs' 
        
        """
        return cls.execute_soa_method(
            method_name='createLaunchFile',
            library='Visualization',
            service_date='2011_02',
            service_name='DataManagement',
            params={'idInfos': idInfos, 'serverInfo': serverInfo, 'userDataAgentInfo': userDataAgentInfo, 'sessionInfo': sessionInfo},
            response_cls=VVITicketsResponse,
        )


class StructureManagementService(TcService):

    @classmethod
    def createVisSC(cls, info: List[CreateVisSCInfo]) -> CreateVisSCResponse:
        """
        This operation takes a list of ConfigurationContext/top line object pairs and creates a VisStructureContext
        object based on that input. The user may optionally supply a list of occurrences in the form of UID chains and
        a file reference for the static PLMXML representation of the configuration. If an occurrence list or a static
        structure file are supplied they will be set as properties on the VisStructureContext object. The list of
        occurrences can be used to populate/expand any BOMWindows that are subsequently created using the output
        VisStructureContext object.
        
        Use cases:
        When the user desires to create a single persistent object that records a particular configuration recipe and
        the caller already has the component objects that make up the configuration. This case might occur if the
        configuration elements of a BOMWindow were captured but the BOMWindow was then deleted. This is often the case
        when using the Teamcenter Thin Client.
        
        
        The createVisSC operation requires input configuration objects and their top lines. Therefore, these objects
        must have been obtained based on some previous configuration scenario.
        """
        return cls.execute_soa_method(
            method_name='createVisSC',
            library='Visualization',
            service_date='2011_02',
            service_name='StructureManagement',
            params={'info': info},
            response_cls=CreateVisSCResponse,
        )

    @classmethod
    def createVisSCsFromBOMs(cls, info: List[CreateVisSCsFromBOMsInfo]) -> CreateVisSCsFromBOMsResponse:
        """
        This operation takes a list of BOMLines (the occurrences list) and returns the VisStructureContext objects
        representing the configuration state of the BOMWindow (referred to as the configuration recipe). This
        configuration includes:
        - The occurrence UID chains for the input/selected BOMLines up to but not including the top line. 
        - Optional IMANFile reference to the PLMXML static representation of the BOMWindow.
        
        
        
        This service supports both the interoperation of selected BOMLines from the Teamcenter Rich Client to
        Teamcenter Visualization and also the capture/persistence of the configuration recipe for a particular
        BOMWindow. The occurrence list records the selected BOMLines at the time of interoperation and can be used in
        later operations to populate/expand a BOMWindow with those same occurrences.
        
        Use cases:
        When the user desires to create a persistent object that records the configuration recipe of a particular
        BOMWindow. The resulting VisStructureContext object would assumedly be used to later reconstruct a BOMWindow
        with the same configuration recipe and the recorded occurrence chains would be used to populate/expand the
        constructed BOMWindow with specific BOMLines. For example, this operation is used when sending selected
        BOMLines from the Structure Manager to Teamcenter Visualization and also to capture the configuration recipe
        for storage in Vis Sessions.
        
        Visualization pruned launch use case
        - User opens a structure in Structure Manager (SM)/Multi Structure Manager (MSM)/Manufacturing Process Planner
        (MPP), and configures it
        - User selects some lines they want to send to visualization as a pruned structure
        - System calls createVisSCsFromBOMs to record the selected lines and the configuration of the BOM to send
        
        
        
        The createVisSCsFromBOMs operation is called with input BOMLines from an existing BOMWindow. Therefore, the
        BOMWindow must have already been created and populated with at least a top line.
        """
        return cls.execute_soa_method(
            method_name='createVisSCsFromBOMs',
            library='Visualization',
            service_date='2011_02',
            service_name='StructureManagement',
            params={'info': info},
            response_cls=CreateVisSCsFromBOMsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.Visualization._2010_09.DataManagement import Snapshot3DStructureFilesInput, GatherSnapshot3DListResponse, FindProductViewForNodesResultRespose, NewSnapshot3DInput, Snapshot3DUpdateStructureFilesResponse, CreateSnapshot3DResponse, Snapshot3DInfoResponse, GatherSnapshot3DInput, Snapshot3DInfoInput, Snapshot3DUpdateInput, Snapshot3DUpdateResponse, FindNodesInProductViewResultResponse, SearchCriteria
from tcsoa.gen.Internal.Visualization._2010_09.StructureManagement import CreateVisSCInfo, CreateVisSCResponse
from typing import List
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def getSnapshot3DInfo(cls, snapshot3DInputList: List[Snapshot3DInfoInput]) -> Snapshot3DInfoResponse:
        """
        This service is used by the visualization integrations in order to gather the information needed to open a
        Product View Dataset and apply it to recreate a 3D scene.  The service gets individual SnapshotViewData Dataset
        information for a list of Product View Datasets of interest including all the named reference file information,
        all the objects related to the Dataset by GRM or named reference, and all the visible lines.  This information
        is used by the viewer to open up the appropriate structure, expand the structure and load the visible lines for
        the Product View, and recreate the 3D scene by applying the viewFile PLMXML.
        
        The client needs to first find the SnapshotViewData Dataset, so the tag can be added to the
        'Snapshot3DInfoInput'.  One or more SnapshotViewData Dataset tags can be added to the vector to get information
        on many Product Views. If more than one Dataset tag is used, it is important to use unique 'clientId' strings
        in the input since they are used as keys in the return output map. If the same 'clientId' string is used,
        information will be lost.
        
        The related objects are used to identify the top line of the structure (e.g. VisTopLevelRef), and to get the
        configuration of the structure to open (e.g. VisStructureContext object).
        
        The visible lines are used to expand the structure and load the appropriate occurrences in order to recreate
        the 3D scene.  This consists of an array of clone stable UIDs or absolute occurrence UIDs depending on the
        version of the Product View model.  The output data will indicate the visible line type in the
        'Snapshot3DVisibleLines' using the uidtype. If the type is a clone stable UID chain, this is a / delimited
        string that represents a path from the root of the structure to the visible line of interest. This list will be
        obtained by processing the structure PLMXML file.
        
        The file information provides the core files that need to be opened by the viewer (e.g. the viewFile PLMXML
        and/or the structure file PLMXML).  The viewFile is used to recreate the 3D scene, and the structure file can
        be used to open the static structure for the Product View to recreate the exact same scene that was saved
        originally.
        
        Use cases:
        This service helps provide the viewer with information to load Product View Datasets.  There are 3 main use
        cases for loading Product Views, each described in more detail below.
        
        Load Product View:
        - In this case the static structure for the visible lines and their descendants is loaded from the Product View
        data model via the structure PLMXML file.  This returns the Product View to the exact as saved state.
        - Load dynamic Product View using stored configuration rules. In this case the structure configuration
        information stored in the Product View data model that represents the configuration of the structure when the
        Product View was saved is used to reconfigure the latest structure with the same configuration rules before the
        scene is recreated.
        - Load dynamic Product View using current configuration rules. In this case the current BOM window
        configuration established in the Structure Manager (SM), Multi Structure Manager (MSM), or Manufacturing
        Process Planner (MPP) application is the structure used to apply the Product View on.
        
        
        
        Load static Product View:
        - User selects one or more Product View objects (SnapShotViewData Datasets) in My Teamcenter and sends those to
        open in visualization.
        - The visualization client gets the list of Datasets to open, builds up the 'Snapshot3DInfoInput' structure for
        each Dataset, and calls the 'getSnapshot3DInfo' operation for the Product Views.
        - The system reads the data model and returns the visible lines list, the named references, and related
        objects.  
        - The viewer downloads the files of interest from the Dataset (e.g. structure PLMXML file, viewFile PLMXML, vpl
        markup layers).  
        - The viewer loads the structure PLMXML file, loads the markup layers (vpl files), and then merges in the
        viewFile PLMXML to recreate the 3D scene.
        
        
        
        Load dynamic Product View using stored configuration rules:
        - User selects one or more Product View objects (SnapShotViewData Datasets) in My Teamcenter and sends those to
        open in visualization.
        - The visualization client gets the list of Datasets to open, builds up the 'Snapshot3DInfoInput' structure for
        each Dataset, and calls the 'getSnapshot3DInfo' operation for the Product Views.
        - The system reads the data model and returns the visible lines list, the named references, and related
        objects.  
        - The viewer downloads the files of interest from the Dataset (structure PLMXML file, viewFile PLMXML, vpl
        markup layers).  
        - The viewer uses the related objects information to get the top line and the configuration information for the
        structure via the VisStructureContext object, and calls the 'createBOMsFromRecipes' operation to configure the
        structure properly.
        - The viewer uses the visible line information to expand the BOM structure and fetch BOMLines.  If clone stable
        occurrence id chains are the type of visible lines returned, the Viewer starts at the top of the structure and
        uses the 'expandPSOneLevel' operation to expand the structure and load it.  If the absolute occurrences are the
        type of visible lines returned or the preference to prune the structure during Product View load is active, the
        viewer uses the 'expandPSFromOccurrenceList' operation to expand the structure and fetch the visible BOMLines.
        - Once the configured structure is loaded and expanded, the viewer loads the markup layers (vpl files), and
        then merges in the viewFile PLMXML to recreate the 3D scene.
        
        
        
        Load dynamic Product View using current configuration rules:
        - User sends a structure to Structure Manager (SM), Multi Structure Manager (MSM), or Manufacturing Process
        Planner (MPP) and configures the structure with revision rules, variant rules, effectivity, etc
        - The user brings up the Product View gallery and finds Product Views of interest (which will eventually invoke
        the 'gatherSnapshot3DList' operation). 
        - The user selects some Product Views and sends those to visualization.  For standalone visualization the
        VisStructureContext sent with the launch file is used to configure the BOMWindow to the current configuration
        from the launching application.  For embedded visualization, the current BOMWindow configuration is used.
        - The visualization client gets the list of Datasets to open, builds up the 'Snapshot3DInfoInput' structure for
        each Dataset, and calls the 'getSnapshot3DInfo' operation for the Product Views.
        - The system reads the data model and returns the visible lines list, the named references, and related
        objects.  
        - The viewer downloads the files of interest from the Dataset (structure PLMXML file, viewFile PLMXML, vpl
        markup layers).  
        - The viewer uses the visible line information to expand the BOM structure and fetch BOMLines if they are not
        already loaded.  If clone stable occurrence ID chains are the type of visible lines returned, the Viewer starts
        at the top of the structure and uses the 'expandPSOneLevel' operation to expand the structure and load it.  If
        the absolute occurrences are the type of visible lines returned or the preference to prune the structure during
        Product View load is active, the viewer uses the 'expandPSFromOccurrenceList' operation to expand the structure
        and fetch the visible BOMLines.
        - Once the configured structure is loaded and expanded, the viewer loads the markup layers (vpl files), and
        then merges in the viewFile PLMXML to recreate the 3D scene.
        
        """
        return cls.execute_soa_method(
            method_name='getSnapshot3DInfo',
            library='Internal-Visualization',
            service_date='2010_09',
            service_name='DataManagement',
            params={'snapshot3DInputList': snapshot3DInputList},
            response_cls=Snapshot3DInfoResponse,
        )

    @classmethod
    def updateSnapshot3D(cls, snapshot3DUpdateInputList: List[Snapshot3DUpdateInput]) -> Snapshot3DUpdateResponse:
        """
        This service updates the Product View data model (or portions of it if desired) consisting of a
        SnapShotViewData Dataset and associated files and relationships.   The call is bulk based and can thus update
        multiple Datasets at a time.  Inputs include the list of Datasets to update and for each Dataset the
        'attachToBOMLine' (A reference to BOMLine to which snapshot Dataset is to be attached), a Boolean
        'createStructureFile' that controls whether or not to create the structure PLMXML file, a Boolean
        'updateVisibleLinesList' along with a 'visibleLinesList' structure used to force the visible line list to be
        updated for Product Views, and the 'namedRefFileInfoList' containing all the file information for the named
        references that are to be uploaded as part of the update.
        
        This service can be used in three different ways:
        
        - Complete update operation. 
        - In conjunction with 'createSnapshot3D' to optimize the performance of initial Product View creation.
        - In conjunction with 'updateSnapshot3DStructureFiles' to optimize performance for Product View updates.
        
        
        
        Use cases:
        Complete update operation:
        - User selects one or more Product View Datasets in Teamcenter and sends those to open in visualization.
        - The visualization client calls the 'getSnapshot3DInfo' service with product view Dataset object as input
        reads the model and returns the visible lines list, the named references, and related objects.  The viewer then
        opens the appropriate product structure, configures it, and applies the Product View by loading the view file
        to recreate the 3D scene.
        - The user modifies the 3D scene by editing / removing / adding markup layers, changing visibility, adjusting
        material settings and colors, repositioning parts, and other common functions supported by Teamcenter
        visualization. 
        - User checks out the Dataset and then invokes the Product View update action in the viewer.
        - Client gathers the list of visible BOMLines for the 3D scene (if changed).
        - Client gathers the attach to BOMLine as the save location for the Product View (if changed).
        - Client exports the Product View files locally that capture the view state data: a viewFile (*.plmxml), a
        thumbnail image (*.jpg, *.cgm), 0..n markup layers (*.vpl) optional, a preview image (*.*) optional, and a 3D
        Geometry Asset file (asset) optional.  From these files and knowledge of the model, the client builds up the
        namedRefFileInfoList for the SnapShotViewData Dataset needed for uploading the files.
        - Client calls the 'updateSnapshot3D' operation and passes the following as input (for each Product View to
        update): snapshot3DDataset tag, 'attachToBOMLine' if changed, 'createStructureFile' = True,
        'updateVisibleLinesList' = False, 'namedRefFileInfoList' containing all the updated files information.
        - Teamcenter/Server updates the complete Product View data model based on the inputs and creates a new version
        of the SnapShotViewData Dataset.  The complete model is updated per the inputs of this call which may end up
        deleting old files that were attached to the previous version but not specified for this version.
        
        
        
        In conjunction with createSnapshot3D:
        - Client application with visualization capabilities loads a Product Structure and visualizes 3D geometry from
        Teamcenter.  
        - The Client application creates a 3D scene by creating markup layers, adjusting material settings and colors,
        repositioning parts, setting visibility, and other common functions supported by Teamcenter visualization
        applications.
        - Client gathers the list of visible BOMLines for the 3D scene
        - Client gathers the attach to BOMLine as the save location for the Product View
        - Client gathers Dataset info (name, description, etc) from the user
        - Client starts a separate thread that will invoke the 'createSnapshot3D' service and passes the
        'attachToBOMLine', 'datasetInfo', 'visibleLinesList', and True for generating the structure PLMXML file on the
        server.  Client does not pass the files to be uploaded in the Dataset (i.e. the name reference information is
        left out).
        - Teamcenter/Server creates partial (incomplete) Product View data model consisting of a SnapShotViewData
        Dataset, a reference to the top line of the structure, the structure configuration (VisStructureContext), and a
        structure (PLMXML) file.  The core files attached required by the data model (viewFile PLMXML and thumbnail
        image) as well as optional files not yet attached.
        - Client concurrently exports the following files locally that capture the view data: a viewFile (*.plmxml), a
        thumbnail image (*.jpg, *.cgm), 0..n markup layers (*.vpl) optional, a preview image (*.*) optional, and a 3D
        Geometry Asset file (asset) optional.  From these files and knowledge of the model, the client builds up the
        'namedRefFileInfoList' needed for uploading the files.
        - Client calls the 'updateSnapshot3D' operation to upload all the files that attach to the SnapShotViewData
        Dataset and passes the Dataset object UID to update, the BOMLine to attach the Dataset to (if different from
        where attached currently), 'createStructureFile' Boolean set to False (since it was created during the
        'createSnapshot3D' call), 'updateVisibleLinesList' Boolean set to False, 'namedRefFileInfoList' containing all
        the file information for the uploaded files.
        - Teamcenter/Server creates the complete Product View data model.
        
        
        
        In conjunction with updateSnapshot3DStructureFiles:
        - User selects one or more Product View objects in Teamcenter and sends those to open in visualization.
        - The visualization client calls the 'getSnapshot3DInfo' operation for the Product Views, which reads the model
        and returns the visible lines list, the named references, and related objects.  The viewer then opens the
        appropriate product structure, configures it, and applies the Product View by loading the viewFile to recreate
        the 3D scene.
        - The user modifies the 3D scene by editing / removing / adding markup layers, changing visibility, adjusting
        material settings and colors, repositioning parts, and other common functions supported by Teamcenter
        visualization. 
        - User checks out the Dataset and invokes the Product View update action in the viewer.
        - Client gathers the list of visible BOMLines for the 3D scene (if changed).
        - Client gathers the attach to BOMLine as the save location for the Product View (if changed).
        - Client starts a separate thread that will invoke the 'updateSnapshot3DStructureFiles' operation and passes
        the SnapShotViewData Dataset and the visible lines.  This function exports a new structure PLMXML file and
        attaches the new file to the new Dataset version.   
        - Concurrently with the 'updateSnapshot3DStructureFiles' operation, the client exports the Product View files
        locally that capture the view state data: a viewFile (*.plmxml), a thumbnail image (*.jpg, *.cgm), 0..n markup
        layers (*.vpl) <optional>, a preview image (*.*) optional, and a 3D Geometry Asset file (asset) optional . 
        From these files and knowledge of the model, the client builds up the 'namedRefFileInfoList' for the
        SnapShotViewData Dataset needed for uploading the files.
        - Client calls the 'updateSnapshot3D' operation and passes the following as input (for each Product View to
        update): 'snapshot3DDataset' tag, 'attachToBOMLine' if changed, 'createStructureFile' = False,
        'updateVisibleLinesList' = False, 'namedRefFileInfoList' containing all the updated files information.
        - Teamcenter/Server updates the SnapShotViewData Dataset based on the inputs by creating a new version of the
        Dataset.  The complete model is updated per the inputs of this call which may end up deleting old files that
        were attached to the previous version but not specified for this version.
        
        """
        return cls.execute_soa_method(
            method_name='updateSnapshot3D',
            library='Internal-Visualization',
            service_date='2010_09',
            service_name='DataManagement',
            params={'snapshot3DUpdateInputList': snapshot3DUpdateInputList},
            response_cls=Snapshot3DUpdateResponse,
        )

    @classmethod
    def updateSnapshot3DStructureFiles(cls, snapshot3DStructureFilesInput: List[Snapshot3DStructureFilesInput]) -> Snapshot3DUpdateStructureFilesResponse:
        """
        This operation updates the 3D structure files within the set of existing Product Views (SnapShotViewData
        datasets) specified.  The structure PLMXML file is exported based on the structure configuration obtained from
        the visible BOMLine list specified in the input, and it contains only the visible lines and their descendants. 
        The structure PLMXML file is created based on the ExportProductView transfer mode.  The VisStructureContext
        object attached to the model is also updated if needed to match the configuration specified by the input
        bomlines.  
        
        The Product View data model (SnapShotViewData dataset) requires a structure PLMXML file export of all the
        visible lines and their descendents.  Structure PLMXML file generation is expensive from a performance
        perspective.  This service is provided as a means for clients to optimize performance for Product View model
        update by running this operation concurrently with other client operations such as exporting files from the
        viewer.  Leveraging concurrency speeds up the overall update process for Product Views.  
        
        This service can also be leveraged for batch update of the structure PLMXML files for product views for use
        cases where Product Views can be updated offline by some utility if desired.  
        
        
        Use cases:
        There are two main use cases where this service is utilized: 1) Optimized update operation, and 2) Offline
        batch update of product views.  The first use case is an optimization done for performance reasons.  Structure
        PLMXML file generation can be time consuming, thus doing this concurrently with other time consuming operations
        such as viewer file export operations can reduce the overall Product View save times.  The batch update use
        case is done when the structure upon which the Product View is based has been updated, and there is a group of
        product views that needs to be updated due to this structure change.  In this case the service is utilized by
        some utility program that is updating product views in some way.
        
        Optimized update operation:
        - User selects one or more Product View objects in Teamcenter and sends those to open in visualization.
        - The visualization client calls 'getSnapshot3DInfo' for the Product Views, which reads the model and returns
        the visible lines list, the named references, and related objects.  The viewer then opens the appropriate
        product structure, configures it, and applies the Product View by loading the viewFile to recreate the 3D scene.
        - The user modifies the 3D scene by editing / removing / adding markup layers, changing visibility, adjusting
        material settings and colors, repositioning parts, and other common functions supported by Teamcenter
        visualization. 
        - User invokes the Product View update action in the viewer.
        - Client gathers the list of visible bomlines for the 3D scene (if changed).
        - Client gathers the attach to BomLine as the save location for the Product View (if changed).
        - Client starts a separate thread that will invoke the 'updateSnapshot3DStructureFiles' operation and passes
        the SnapShotViewData dataset and the visible lines.  This function exports a new structure PLMXML file and
        uploads the new file to the snapshot dataset.   
        - Concurrently with the 'updateSnapshot3DStructureFiles' operation call, the client exports the Product View
        files locally that capture the view state data: a viewFile (*.plmxml), a thumbnail image (*.jpg, *.cgm), 0..n
        markup layers (*.vpl) optional, a preview image (*.*) optional, and a 3D Geometry Asset file (asset) optional .
         From these files and knowledge of the model, the client builds up the 'namedRefFileInfoList' for the
        SnapShotViewData dataset needed for uploading the files.
        - Client calls the 'updateSnapshot3D' operation and passes the following as input (for each Product View to
        update): 'snapshot3DDataset' tag, 'attachToBOMLine' if changed, 'createStructureFile' = False,
        'updateVisibleLinesList' = False, 'namedRefFileInfoList' containing all the updated files information.
        - Teamcenter/Server updates the SnapShotViewData Dataset based on the inputs. The complete model is updated per
        the inputs of this call which may end up deleting old files.
        
        
        
        Batch update operation:
        - Some process is conducted that determines the structure upon which a Product View is based has been modified
        in some way that affects the product view display.
        - The structure is opened and configured per some external input.
        - The Product Views attached to this structure are determined.
        - For each Product View, the model for the Product View is interrogated, and the visible line list retrieved
        using the 'getSnapshot3DInfo' operation.
        - BomLines are expanded for the visible line list by using something like the 'expandPSFromOccurrenceList'
        operation.
        - The 'snapshot3DStructureFilesInput' structure is built containing the SnapShotViewData Dataset and the
        visible line list for this Dataset.
        - Steps 4 through 6 are repeated for all Product Views to update.
        - The 'updateSnapshot3DStructureFiles' operation call is made.
        - The system updates the structure PLMXML files for each SnapShotViewData Dataset specified on the input.
        
        """
        return cls.execute_soa_method(
            method_name='updateSnapshot3DStructureFiles',
            library='Internal-Visualization',
            service_date='2010_09',
            service_name='DataManagement',
            params={'snapshot3DStructureFilesInput': snapshot3DStructureFilesInput},
            response_cls=Snapshot3DUpdateStructureFilesResponse,
        )

    @classmethod
    def createSnapshot3D(cls, newSnapshot3DInputList: List[NewSnapshot3DInput]) -> CreateSnapshot3DResponse:
        """
        This operation creates the Product View data model (or portions of it if desired) consisting of a
        SnapShotViewData Dataset and associated files and relationships A live BOMWindow must be configured for the
        loaded structure with BOMLines expanded for all the visible lines to be captured in the product view.  These
        visible lines along with a BOMLine where user wishes to attach the Dataset is to be sent as input to the
        operation. Product view Dataset creation information (name and description) should be sent as input to the
        operation as well. Two files are ultimately required by the data model (a viewFile (PLMXML) file, and a
        thumbnail image (jpg, cgm)).  Several optional files can also be specified (a preview image (*.*), motion
        (vfm), markup layers (vpl), and 3D Geometry Asset image (asset)).  These files can first be exported prior to
        calling this service, and the entire model will be created by this service.
        
        The server implementation of this operation will create the structure (PLMXML) file that represents an export
        of the structure for all visible lines and their descendants if specified.  However, creation of the structure
        (PLMXML) file can be very costly from a performance perspective.  As a result, the service has been set up so
        that the caller can initiate this call early in the save process just to create the basics of the model
        including export of structure file before gathering the required files from the viewer (i.e. Thumbnail (jpg,
        cgm), ViewFile (PLMXML), and the optional files) and uploading them.  If this approach is used, the viewer
        files must be saved later using the 'UpdateSnapshot3D' operation.  The operation is set up to handle both
        approaches.
        
        The server implementation of this service creates a VisStructureContext object (or objects) representing the
        configuration of the loaded structure based on the BOPWindow or BOMWindow specified via the BOMLines passed for
        the visible lines. This object is capable of storing configuration information for Product and Process
        structures, including composition structures.  This allows clients to reopen a Product View with the exact same
        structure configuration that was in effect when the Product View was created for all the structure types
        supported by the BVR model.
        
        The service returns the map between client IDs and created Dataset objects which may or may not have all the
        necessary files attached depending on the calling implementation.
        
        Use cases:
        This operation is for clients that have loaded a 3D structure from Teamcenter with a live connection to a
        BOMWindow on the server, with live references to BOMLines within that structure representing the visible lines
        and attach to location.  The visible lines will be recorded as the occurrences that must be loaded to restore
        the Product View, and the attach to location will be the ItemRevision (pointed to by BOMLine) where the user
        wants the Product View stored attached. A new VisTopLevelRef relation is also created between new snapshot
        Dataset and BOMView corresponding to loaded structure.
        
        There are two basic use cases for calling this method: 
        - Create complete model. 
        - Create partial model, upload files later.
        
        
        
        Create complete model:
        - Client application with visualization capabilities loads a Product Structure and visualizes 3D geometry from
        Teamcenter.  
        - The Client application creates a 3D scene by creating markup layers, adjusting material settings and colors,
        temporary repositioning parts, setting visibility, and other common functions supported by Teamcenter
        visualization applications.
        - Client gathers the list of visible BOMLines for the 3D scene.
        - Client gathers the attach to BOMLine as the save location for the Product View.
        - Client gathers Dataset info (name, description, etc) from the user.
        - Client exports the following files locally that capture the view data: a viewFile (*.plmxml), a thumbnail
        image (*.jpg, *.cgm), 0..n markup layers (*.vpl) <optional>, a preview image (*.*) <optional>, and a 3D
        Geometry Asset file (asset) <optional> .  From these files and knowledge of the model, the client builds up the
        named reference information for the SnapShotViewData Dataset needed for uploading the files.
        - Client calls 'createSnapshot3D' operation and passes a reference to BOMLine to which snapshot Dataset is to
        be attached, datasetInfo, visibleLinesList, named reference information, and a Boolean true for generating the
        structure PLMXML file on the server.
        - Teamcenter/Server creates the complete Product View data model per the configuration of the BOPWindow or
        BOMWindow and the inputs.
        
        
        
        Create partial model, upload files later:
        - Client application with visualization capabilities loads a Product Structure and visualizes 3D geometry from
        Teamcenter.  
        - Client application creates a 3D scene by creating markup layers, adjusting material settings and colors,
        repositioning parts, setting visibility, and other common functions supported by Teamcenter visualization
        applications.
        - Client gathers the list of visible BOMLines for the 3D scene.
        - Client gathers the attach to BOMLine as the save location for the Product View.
        - Client gathers Dataset info (name, description, etc) from the user.
        - Client starts a separate thread that will invoke the 'createSnapshot3D' service and passes the
        'attachToBOMLine', 'datasetInfo', 'visibleLinesList', and a Boolean true for generating the structure PLMXML
        file on the server.  Client does NOT pass the files that attach to the Dataset (i.e. the named reference
        information is left out).
        - Teamcenter/Server creates partial (incomplete) Product View data model consisting of a SnapShotViewData
        Dataset, a reference to the top line of the structure, the structure configuration (VisStructureContext), and a
        structure (PLMXML) file.  The core files required by the data model (viewFile PLMXML and thumbnail image) as
        well as optional files are not yet attached.
        - Client concurrently exports the following files locally that capture the view data: a viewFile (*.plmxml), a
        thumbnail image (*.jpg, *.cgm), 0..n markup layers (*.vpl) <optional>, a preview image (*.*) <optional>, and a
        3D Geometry Asset file (asset) <optional> .  From these files and knowledge of the model, the client builds up
        the named reference information for the SnapShotViewData Dataset needed for uploading the files.
        - Client calls the 'updateSnapshot3D' operation to upload all the files that attach to the SnapShotViewData
        Dataset
        - System creates the complete Product View data model.
        
        """
        return cls.execute_soa_method(
            method_name='createSnapshot3D',
            library='Internal-Visualization',
            service_date='2010_09',
            service_name='DataManagement',
            params={'newSnapshot3DInputList': newSnapshot3DInputList},
            response_cls=CreateSnapshot3DResponse,
        )

    @classmethod
    def findProductViewsForNodes(cls, searchCriteria: List[SearchCriteria]) -> FindProductViewForNodesResultRespose:
        """
        Find all product views within the specified scope and returns the product views and the nodes present in the
        product view out of the input nodes.
        """
        return cls.execute_soa_method(
            method_name='findProductViewsForNodes',
            library='Internal-Visualization',
            service_date='2010_09',
            service_name='DataManagement',
            params={'searchCriteria': searchCriteria},
            response_cls=FindProductViewForNodesResultRespose,
        )

    @classmethod
    def gatherSnapshot3DList(cls, input: List[GatherSnapshot3DInput]) -> GatherSnapshot3DListResponse:
        """
        THIS OPERATION IS DEFINED FOR FUTURE USE AND HAS NOT BEEN IMPLEMENTED.
        
        This operation is intended for population of the Product View Gallery with Product Views, displayed as
        thumbnails.  The operation returns only enough information to populate the Product View Gallery (but no more)
        to ensure gallery performance is good.  The input is a list of BOMLines where the SnapshotViewData may be
        attached and GRM relation to be followed to gather snapshots.  The return is a list of SnapshotViewData
        Datasets mapped to the BOMLines sent along with their thumbnail and preview (optional) images as IMANFile
        objects.  This allows the caller to retrieve the files for thumbnail and preview image display without further
        server interaction.  
        
        The gather behavior can be tailored by the client by adjusting the input arguments that specify which type of
        GRM relations to traverse for the specific bomlines passed.  For example, if the VisTopLevelRef is specified
        along with the top bomline and no other criteria, all the SnapshotViewData Datasets captured in the top line
        context will be found, regardless of where they are attached.  However, if IMAN_3D_snap_shot and VisTopLevelRef
        are specified, then it will filter the results and only return the SnapshotViewData Datasets attached to the
        top line.
        
        
        Use cases:
        The primary use case for this service is to implement the logic for displaying Product Views in the Product
        View Gallery behind a single service operation so that all clients can benefit from the same basic business
        logic.  
        
        The logic can be summarized as follows:
        - Get the list of Product Views from attachments panel (Attachments Window) for the selected line.
        - If nothing is returned from the attachments window, get the Product Views related to the selected item
        revision via an IMAN_3D_snap_shot relationship.
        - Filter out the product views that do not have the VisTopLevelRef reference that matches the top line BOMView
        (ensure the top level context matches so the occurrence references resolve).
        
        
        
        The Product View Gallery population use case for this service is a follows:
        
        Product View Gallery population:
        - User sends a structure to Structure Manager (SM), Multi Structure Manager (MSM), or Manufacturing Process
        Planner (MPP), configures the structure, and may do some level of structure expansion.
        - User turns on the Product View gallery to find a list of Product Views to display
        - Client gathers the top BOMLine, and optionally gathers additional BOMLines depending on the desired filtering
        criteria.  Most typically, the top BOMLine and reference type VisTopLevelRef are specified as the inputs to the
        service if it is desired to find all Product Views for this top level context.  However, if selected BOMLines
        and the reference type IMAN_3D_snap_shot are sent, the return is further filtered to only provide those
        snapshots attached to the selected bomline.  The system builds up the 'GatherSnapshot3DInput' structure with
        this filtering information.
        - Client calls the 'gatherSnapshot3DList' operation which then finds all the Product Views that match the
        filter criteria along with their thumbnail and preview images and returns them to the caller.  
        - Teamcenter/Server downloads the thumbnail images from the Teamcenter File Management System (FMS) for the
        gallery display
        - Client updates the gallery after getting the output from the service to include the returned Product Views.
        
        """
        return cls.execute_soa_method(
            method_name='gatherSnapshot3DList',
            library='Internal-Visualization',
            service_date='2010_09',
            service_name='DataManagement',
            params={'input': input},
            response_cls=GatherSnapshot3DListResponse,
        )

    @classmethod
    def getNodesPresentInProductView(cls, searchScope: List[BusinessObject], productView: BusinessObject, nodesToSearch: List[BusinessObject]) -> FindNodesInProductViewResultResponse:
        """
        This function returns the nodes present in the given product view. This SOA will be called once the PVs are
        updated and to show the nodes (out of the input node list) present in the PV.
        """
        return cls.execute_soa_method(
            method_name='getNodesPresentInProductView',
            library='Internal-Visualization',
            service_date='2010_09',
            service_name='DataManagement',
            params={'searchScope': searchScope, 'productView': productView, 'nodesToSearch': nodesToSearch},
            response_cls=FindNodesInProductViewResultResponse,
        )


class StructureManagementService(TcService):

    @classmethod
    def createVisSC(cls, info: List[CreateVisSCInfo]) -> CreateVisSCResponse:
        """
        This operation takes a list of ConfigurationContext/top line object pairs and creates a VisStructureContext
        object based on that input. The user may optionally supply a list of occurrences in the form of UID chains and
        a file reference for the static PLMXML representation of the configuration. If an occurrence list or a static
        structure file are supplied they will be set as properties on the VisStructureContext object. The list of
        occurrences can be used to populate/expand any BOMWindows that are subsequently created using the output
        VisStructureContext object (e.g. see 'expandPSFromOccurrenceList').
        
        Use cases:
        When the user desires to create a single persistent object that records a particular configuration recipe and
        the caller already has the component objects that make up the configuration. This case might occur if the
        configuration elements of a BOMWindow were captured but the BOMWindow was then deleted. This is often the case
        when using the Teamcenter Thin Client.
        
        The 'createVisSC' operation requires input configuration objects and their top lines. Therefore, these objects
        must have been obtained based on some previous configuration scenario.
        """
        return cls.execute_soa_method(
            method_name='createVisSC',
            library='Internal-Visualization',
            service_date='2010_09',
            service_name='StructureManagement',
            params={'info': info},
            response_cls=CreateVisSCResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ServiceData(TcBaseObj):
    """
    System class - needs reverse engineering!
    """
    plain: List[str] = ()
    updated: List[str] = ()
    deleted: List[str] = ()
    created: List[str] = ()
    partialErrors: List[ErrorStack] = ()
    modelObjects: TagMap = None


@dataclass
class ErrorStack(TcBaseObj):
    """
    System class - needs reverse engineering!
    """
    clientId: str = ''
    clientIndex: str = ''
    associatedObject: BusinessObject = None
    errorValues: List[ErrorValue] = ()


@dataclass
class ErrorValue(TcBaseObj):
    """
    System class - needs reverse engineering!
    """
    message: str = ''
    code: int = 0
    level: int = 0


@dataclass
class PartialErrors(TcBaseObj):
    """
    System class - needs reverse engineering!
    """
    pass


@dataclass
class Preferences(TcBaseObj):
    """
    System class - needs reverse engineering!
    """
    pass


@dataclass
class TypeSchema(TcBaseObj):
    """
    System class - needs reverse engineering!
    """
    pass


@dataclass
class ModelSchema(TcBaseObj):
    """
    System class - needs reverse engineering!
    """
    pass


TagMap = Dict[str, BusinessObject]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from typing import List
from tcsoa.utils import TcService


class SimulationProcessManagementService(TcService):

    @classmethod
    def importSimulationObjects(cls, processHierarchy: str, workingDirectory: str, itemID: str, itemKey: str, revisionID: str, xmlFileName: str, itemCreationSetting: str, datasetCreationSetting: str) -> bool:
        """
        This internal operation will import the output files generated from execution of simulation tool launch process
        into Teamcenter objects. Based on the input values of parameters itemCreationSetting and
        datasetCreationSetting, ItemRevision and Dataset objects will be created or modified to store the imported
        simulation tool launch data.
        
        It is pre-requisite that Simulation Administrator or user with DBA privileges must configure the simulation
        tool and store it in XML format as a named reference with the Dataset name specified in the preference
        CAE_simulation_tool_config_dsname.
        
        To use this operation, the user should have either a simulation_author or rtt_author license.
        
        
        Use cases:
        Use Case 1: Import Simulation Objects with "As_Needed" Item Creation Setting
        When this operation is executed with Item creation setting "As_Needed", no new Teamcenter Item will be created
        if there exists one as per the process output configuration. If found none, new Item will be created as per the
        process output configuration.
        
        Use Case 2: Import Simulation Objects with "Always" Item Creation Setting
        When this operation is executed with Item creation setting "Always", new Teamcenter Item will be created as per
        the process output configuration.
        
        Use Case 3: Import Simulation Objects with "Never" Item Creation Setting
        When this operation is executed with Item creation setting "Never" and if no existing Item is found as per the
        process output configuration, no new Teamcenter Item will be created.
        
        Use Case 4: Import Simulation Objects with "As_Needed" Dataset Creation Setting
        When this operation is executed with Dataset creation setting "As_Needed", no new Teamcenter Dataset will be
        created if there exists one as per the process output configuration. If found none, new Dataset will be created
        as per the process output configuration.
        
        Use Case 5: Import Simulation Objects with "Always" Dataset Creation Setting
        When this operation is executed with Dataset creation setting "Always", new Teamcenter Dataset will be created
        as per the process output configuration.
        
        Use Case 6: Import Simulation Objects with "Never" Dataset Creation Setting
        When this operation is executed with Dataset creation setting "Never" and if no existing Dataset is found as
        per the process output configuration, no new Teamcenter Dataset will be created.
        """
        return cls.execute_soa_method(
            method_name='importSimulationObjects',
            library='Internal-Cae',
            service_date='2012_02',
            service_name='SimulationProcessManagement',
            params={'processHierarchy': processHierarchy, 'workingDirectory': workingDirectory, 'itemID': itemID, 'itemKey': itemKey, 'revisionID': revisionID, 'xmlFileName': xmlFileName, 'itemCreationSetting': itemCreationSetting, 'datasetCreationSetting': datasetCreationSetting},
            response_cls=bool,
        )


class StructureManagementService(TcService):

    @classmethod
    def generateNodeXML(cls, bomlines: List[BOMLine], domain: str) -> str:
        """
        This internal operation generates a NodeXML representation of the BOMLine object(s) identified in the parameter
        and the selected domain defined as LOV under StructureMap Domains  in  BMIDE. NodeXML is XML-based and is most
        commonly used in the process for creating Data Map and StructureMap rules. The generated NodeXML consists of
        the visible attributes and their values associated with the Item, ItemRevision, related Form objects and the
        BOMLine. The NodeXML syntax is in compliance with the schema defined in tcsim_sm_node.xsd, located in TC_DATA.
        
        For more details on building and executing Data Map and StructureMap rules, please refer to the "Creating
        StructureMap rules" section in the Simulation Process Management Guide->Using StructureMaps chapter of the
        Teamcenter documentation.
        
        
        Use cases:
        Use Case 1: Generate a NodeXML for a selected BOMLine
        When a BOM structure is present or accessible, select one or more BOMLine objects from the BOM structure and
        the selected StructureMap Domain and invoke the generateNodeXML operation to generate the NodeXML
        interpretation of the selected BOMLine objects.
        """
        return cls.execute_soa_method(
            method_name='generateNodeXML',
            library='Internal-Cae',
            service_date='2012_02',
            service_name='StructureManagement',
            params={'bomlines': bomlines, 'domain': domain},
            response_cls=str,
        )



from __future__ import annotations

from tcsoa.gen.ProjectManagement._2007_01.ScheduleManagement import CreateBaselineContainer
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ScheduleManagementService(TcService):

    @classmethod
    def createNewBaselines(cls, createBaselineContainer: List[CreateBaselineContainer], runInBackground: bool) -> ServiceData:
        """
        This operation creates new schedule baselines. In case of background mode, this operation files an asynchronous
        request to create the baselines and releases the client immediately so that the user can perform other
        operation.
        """
        return cls.execute_soa_method(
            method_name='createNewBaselines',
            library='ProjectManagement',
            service_date='2016_04',
            service_name='ScheduleManagement',
            params={'createBaselineContainer': createBaselineContainer, 'runInBackground': runInBackground},
            response_cls=ServiceData,
        )

    @classmethod
    def createNewBaselinesAsync(cls, createBaselineContainer: List[CreateBaselineContainer]) -> None:
        """
        This operation creates new 'schedule' baselines. This operation runs asynchronously in its own server in the
        background.
        """
        return cls.execute_soa_method(
            method_name='createNewBaselinesAsync',
            library='ProjectManagement',
            service_date='2016_04',
            service_name='ScheduleManagement',
            params={'createBaselineContainer': createBaselineContainer},
            response_cls=None,
        )



from __future__ import annotations

from typing import Dict, List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateWorkflowInput(TcBaseObj):
    """
    input structure to create workflow process.
    
    :var processName: Name of the workflow process to be created.
    :var processDescription: Longer description of the purpose of the workflow process to be created.
    :var processTemplateName: Name of the process template. This must be a valid, existing process template.
    :var attachments: List of atachments representing either target or reference objects that will be added at process
    creation time. List may consist of target attachments or reference attachments or a mixture of both.
    :var attachmentTypes: Identifies the types of attachments listed in attachment.  Valid types include
    EPM_target_attachment (target attachment) and EPM_reference_attachment (reference attachment). There is a
    one-to-one correspondence between the attachment types on this list and the attachements list.
    :var processAssignmentList: Name of the process assignment list.
    :var additonalData: Map of property name( key ) and the property values( value )  in string format. This input is
    currently not processed and  
    added for future use.
    """
    processName: str = ''
    processDescription: str = ''
    processTemplateName: str = ''
    attachments: List[str] = ()
    attachmentTypes: List[int] = ()
    processAssignmentList: str = ''
    additonalData: KeyValuePair = None


"""
Structure for passign data in form of key and values.
"""
KeyValuePair = Dict[str, List[str]]



from tcsoa.gen.Internal_ImportExport._2010_04.services import L10NImportExportService as imp0
from tcsoa.gen.Internal_ImportExport._2018_06.services import L10NImportExportService as imp1
from tcsoa.gen.Internal_ImportExport._2017_11.services import FileImportExportService as imp2
from tcsoa.gen.Internal_ImportExport._2017_05.services import FileImportExportService as imp3
from tcsoa.utils import TcService


class L10NImportExportService(TcService):
    exportObjectsForTranslation = imp0.exportObjectsForTranslation
    exportObjectsForTranslation2 = imp1.exportObjectsForTranslation2
    filterObjectsForTranslation = imp0.filterObjectsForTranslation
    importTranslations = imp0.importTranslations


class FileImportExportService(TcService):
    importDocumentAsync = imp2.importDocumentAsync
    importDocumentOffline = imp2.importDocumentOffline
    importSpecAsync = imp3.importSpecAsync



from __future__ import annotations

from tcsoa.gen.Manufacturing._2008_06.TimeManagement import TimeAnalysisInputs, AllocatedTimeResponse, TimeAnalysisRollupResponse, AllocatedTime
from tcsoa.gen.BusinessObjects import BusinessObject, Folder
from tcsoa.gen.Manufacturing._2008_06.DataManagement import CreateOrUpdateMEActivityFolderResponse, MEActivityFolderInfo, MENXObjectInfo, CreateOrUpdateMENXObjectResponse
from tcsoa.gen.Manufacturing._2008_06.Core import FindCheckedOutsInStructureResponse
from typing import List
from tcsoa.utils import TcService


class TimeManagementService(TcService):

    @classmethod
    def timeAnalysisRollup(cls, inputs: TimeAnalysisInputs) -> TimeAnalysisRollupResponse:
        """
        Calculates the total time for each activity category under a requested bop line.
        An additional calculation is all the run time propertie related to the bop line time calculations
        such as total time and duration time.
        """
        return cls.execute_soa_method(
            method_name='timeAnalysisRollup',
            library='Manufacturing',
            service_date='2008_06',
            service_name='TimeManagement',
            params={'inputs': inputs},
            response_cls=TimeAnalysisRollupResponse,
        )

    @classmethod
    def allocatedTimeRollUp(cls, object: AllocatedTime) -> AllocatedTimeResponse:
        """
        Calculates the allocated time for each requested bop line.
        """
        return cls.execute_soa_method(
            method_name='allocatedTimeRollUp',
            library='Manufacturing',
            service_date='2008_06',
            service_name='TimeManagement',
            params={'object': object},
            response_cls=AllocatedTimeResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def createOrUpdateMEActivityFolders(cls, activityInfos: List[MEActivityFolderInfo]) -> CreateOrUpdateMEActivityFolderResponse:
        """
        Allows the user to create and/or update a MEActivityFolder.  If the given MEActivity object
        exists but the input attribute values that differ from those already set, an attempt is made
        to update the values if possible.
        If multiple level of sub activities are to be created those activities can be passed in as
        the objects if they already exist in database. The created folder and updated folders are returned
        to the client through the createdObjects and updatedObject list of the service data respectively.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateMEActivityFolders',
            library='Manufacturing',
            service_date='2008_06',
            service_name='DataManagement',
            params={'activityInfos': activityInfos},
            response_cls=CreateOrUpdateMEActivityFolderResponse,
        )

    @classmethod
    def createOrUpdateMENXObjects(cls, meObjectInfos: List[MENXObjectInfo], container: Folder) -> CreateOrUpdateMENXObjectResponse:
        """
        Allows the user to create and/or update a MENXObject. If the given MENXObject object
        exists but the input attribute values that differ from those already set, an attempt
        is made to update the values if possible.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateMENXObjects',
            library='Manufacturing',
            service_date='2008_06',
            service_name='DataManagement',
            params={'meObjectInfos': meObjectInfos, 'container': container},
            response_cls=CreateOrUpdateMENXObjectResponse,
        )


class CoreService(TcService):

    @classmethod
    def findCheckedOutsInStructure(cls, searchScope: List[BusinessObject]) -> FindCheckedOutsInStructureResponse:
        """
        Finds all the checked out items in the objects.
        """
        return cls.execute_soa_method(
            method_name='findCheckedOutsInStructure',
            library='Manufacturing',
            service_date='2008_06',
            service_name='Core',
            params={'searchScope': searchScope},
            response_cls=FindCheckedOutsInStructureResponse,
        )



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.BusinessObjects import SavedSearch
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class Awp0UpdateAdvancedSavedSearchInput(TcBaseObj):
    """
    Input structure for updateAdvancedSavedSearch operation.
    
    :var stringValueInputKeyValuePairs: A map (string, string) to hold string value nputs. Valid keys are followings:
    - "savedSearchUid" The uid of saved search to update.
    - "savedSearchName" The saved search name.
    - "clientId" This unique ID is used to identify return data elements and partial errors associated with this input
    structure.
    
    
    :var boolValueInputKeyValuePairs: A map (string, bool) to hold bool value inputs. Valid keys are followings:
    - "shareSavedSearch" Allow other users to view and execute the saved search. If true, other users can see and
    execute the saved search; otherwise not visible to other users. Only applicable for owning user.
    - "favorite" Favorite saved search of current user. If true, it&rsquo;s a favorite saved search of current user;
    otherwise a regular one.
    - "pinToHome" Pin to home for current user. If true, pin the saved search to home; otherwise not pin to home.
    - "override" If false and a SavedSearch object having same name owned by current user already exists, then partial
    error returned. If true and a SavedSearch object having same name owned by current user already exists then delete
    the existing SavedSearch object and create a new one. Only applicable for owning user.
    
    """
    stringValueInputKeyValuePairs: StringValueInputKeyValueMap = None
    boolValueInputKeyValuePairs: BoolValueInputKeyValueMap = None


@dataclass
class SavedSearchCriteria(TcBaseObj):
    """
    A strucuture represents a saved search critieria.
    
    :var criteriaName: Name of critiera. e.g. "fnd0CurrentLocationCode"
    :var criteriaValue: Value of criteria. e.g. "us"
    :var criteriaDisplayValue: Display value of critiera. e.g. "United States"
    """
    criteriaName: str = ''
    criteriaValue: str = ''
    criteriaDisplayValue: str = ''


@dataclass
class Awp0AdvancedSavedSearchResponse(TcBaseObj):
    """
    Contians a list of newly created saved searches and the service data containing partial errors if any.
    
    :var savedSearchList: A List of SavedSearch objects created or modified by a service operation.
    :var serviceData: Returned service data.
    """
    savedSearchList: List[SavedSearch] = ()
    serviceData: ServiceData = None


@dataclass
class Awp0CreateAdvancedSavedSearchInput(TcBaseObj):
    """
    Input structure for createAdvancedSavedSearch operation.
    
    :var stringValueInputKeyValuePairs: A map (string, string) to hold string value inputs, valid keys are followings:
    - "savedSearchName" The name of saved search.
    - "referencingSavedQuery" The saved query this saved search is created based off.
    - "targetFolder" The target Folder the saved search will be created in.
    - "clientId" This unique ID is used to identify return data elements and partial errors associated with this input
    structure.
    
    
    :var boolValueInputKeyValuePairs: A map (string, bool) to hold bool value inputs, valid keys are followings:
    - "shareSavedSearch" Allow other users to view and execute the saved search.  If true, other users can see and
    execute the saved search; otherwise not visible to other users.
    - "favorite" Favorite saved search of current user. If true, it&rsquo;s a favorite saved search of current user;
    otherwise a regular one.
    - "pinToHome" Pin to home for current user. Valid values are:If true, pin the saved search to home; otherwise not
    pin to home.
    - "override" Valid values are: True or False. If false and a SavedSearch object having same name owned by current
    user already exists, then partial error returned. If true and a SavedSearch object having same name owned by
    current user already exists then delete the existing SavedSearch object and create a new one.
    
    
    :var savedSearchCriteria: The search critiera (user inputs) used to create the saved search, which will be used
    when executing the saved search.
    """
    stringValueInputKeyValuePairs: StringValueInputKeyValueMap = None
    boolValueInputKeyValuePairs: BoolValueInputKeyValueMap = None
    savedSearchCriteria: List[SavedSearchCriteria] = ()


"""
Key value pair represents bool value input.
"""
BoolValueInputKeyValueMap = Dict[str, bool]


"""
Key value pair represents string value input.
"""
StringValueInputKeyValueMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Cad._2007_01.StructureManagement import CreateBOMWindowsResponse
from tcsoa.gen.Cad._2019_06.StructureManagement import CreateWindowsInfo3
from typing import List
from tcsoa.utils import TcService


class StructureManagementService(TcService):

    @classmethod
    def createOrReConfigureBOMWindows(cls, info: List[CreateWindowsInfo3]) -> CreateBOMWindowsResponse:
        """
        Creates a list of windows and set the respective input ItemRevision as the top line. This operation can be used
        to set, update and reconfigure BOMWindow using multiple saved variant rules or single stored option set to the
        window. This will provide the ability to specify and apply more configuration for the BOMWindow. For setting
        Product Configurator authored variant rule on the window, value of preference "PSM_enable_product_configurator"
        must be true. It can be used to set certain window property, if sent as a part of input. It can be used to
        create the BOMLine for input to Expand Product Structure services. All BOMLine objects under this window are
        unpacked. To use the Teamcenter default unitNo or use your input RevisionRule with no changes, you must set
        unitNo to -1 in RevisionRuleEntryProps::unitNo. If it is not specified, your input rule will function as a
        modified/transient revision rule with a unitNo of 0.
        """
        return cls.execute_soa_method(
            method_name='createOrReConfigureBOMWindows',
            library='Cad',
            service_date='2019_06',
            service_name='StructureManagement',
            params={'info': info},
            response_cls=CreateBOMWindowsResponse,
        )



from __future__ import annotations

from tcsoa.gen.Cae._2013_12.SimulationProcessManagement import InputObjectsStructure2
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class SimulationProcessManagementService(TcService):

    @classmethod
    def launchSimulationTool2(cls, inputObjects: List[InputObjectsStructure2], toolName: str, launchType: str, itemCreationOption: str, datasetCreationOption: str, plmxmlExportFileName: str, workingDirectory: str, runtimeArguments: List[str]) -> ServiceData:
        """
        This operation launches the simulation tool as per the pre-defined configuration. The configuration contains
        details about the tool launch type, export and import rules for the files, notification, and cleanup. The
        inputs to the operation is an array of InputObjectStructure2, each containing one or more business objects,
        name of the tool to be launched and the launch type. The launch type could be LOCAL or REMOTE. Based on the
        input values of parameters itemCreationOption and datasetCreationOption, ItemRevision and Dataset objects are
        created or modified.
        
        It is a pre-requisite that the Simulation Administrator or user with DBA privileges must configure the
        simulation tool and store it in XML format as a named reference with the Dataset name specified in the
        preference CAE_simulation_tool_config_dsname.
        
        To use this operation, the user should have either a simulation_author or rtt_author license. 
        
        
        Use cases:
        Use Case 1: Launch Simulation Tool with As Needed Item Creation Option
        When this operation is executed with itemCreationOption "As Needed", no new Teamcenter Item will be created if
        there exists one as per the process output configuration. If found none, new Item will be created as per the
        process output configuration.
        
        Use Case 2: Launch Simulation Tool with Always Item Creation Option
        When this operation is executed with itemCreationOption "Always", new Teamcenter Item objects will be created
        as per the process output configuration.
        
        Use Case 3: Launch Simulation Tool with Never Item Creation Option
        When this operation is executed with itemCreationOption "Never" and if no existing Item is found as per the
        process output configuration, no new Teamcenter Item objects will be created.
        
        Use Case 4: Launch Simulation Tool with As Needed Dataset Creation Option
        When this operation is executed with datasetCreationOption "As Needed", no new Teamcenter Dataset will be
        created if there exists one as per the process output configuration. If found none, new Dataset will be created
        as per the process output configuration.
        
        Use Case 5: Launch Simulation Tool with Always Dataset Creation Option
        When this operation is executed with datasetCreationOption "Always", new Teamcenter Dataset objects will be
        created as per the process output configuration.
        
        Use Case 6: Launch Simulation Tool with Never Dataset Creation Option
        When this operation is executed with datasetCreationOption "Never" and if no existing Dataset is found as per
        the process output configuration, no new Teamcenter Dataset objects will be created.
        """
        return cls.execute_soa_method(
            method_name='launchSimulationTool2',
            library='Cae',
            service_date='2013_12',
            service_name='SimulationProcessManagement',
            params={'inputObjects': inputObjects, 'toolName': toolName, 'launchType': launchType, 'itemCreationOption': itemCreationOption, 'datasetCreationOption': datasetCreationOption, 'plmxmlExportFileName': plmxmlExportFileName, 'workingDirectory': workingDirectory, 'runtimeArguments': runtimeArguments},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FileTicket(TcBaseObj):
    """
    Structure containing information related to file ticket.
    
    :var ticket: The FMS file ticket.
    :var fileName: The original file name.
    """
    ticket: str = ''
    fileName: str = ''


@dataclass
class ValidateScopeFlowsConsistencyResponse(TcBaseObj):
    """
    Response structure for validateScopeFlowsConsistency service.
    
    :var data: A list of ConsistencyData. Each element in the list contains indication whether the data is consistent
    and information about the in-cosistencies (cycles), if found.
    :var fileTicket: An FMS file ticket. The file contains report about the cycles found by the algorithm.
    :var serviceData: Service data.
    """
    data: List[ConsistencyData] = ()
    fileTicket: FileTicket = None
    serviceData: ServiceData = None


@dataclass
class ConsistencyData(TcBaseObj):
    """
    Structure containing information related to cycles.
    
    :var isConsistent: If true, no cycles are found under the given scope.
    :var cycles: The list of CycleData. Each element in the list contains data about a single cycle.
    """
    isConsistent: bool = False
    cycles: List[CycleData] = ()


@dataclass
class CycleData(TcBaseObj):
    """
    Structure containing information related to a specefic cycle identified.
    
    :var objects: The list of operations/processes that participate in a cycle that. The list contains business objects
    of type Mfg0BvrOperation and Mfg0BvrProcess.
    :var flows: The list of scope flows which participates in the cycle. These objects are of type Mfg0BvrScopeFlow.
    """
    objects: List[BusinessObject] = ()
    flows: List[BusinessObject] = ()



from __future__ import annotations

from tcsoa.gen.ProjectManagementAw._2015_07.ScheduleManagementAw import TaskDeliverableContainer, CreateTaskDeliverablesResponse
from typing import List
from tcsoa.utils import TcService


class ScheduleManagementAwService(TcService):

    @classmethod
    def createTaskDeliverables(cls, taskDeliverableData: List[TaskDeliverableContainer]) -> CreateTaskDeliverablesResponse:
        """
        This operation takes in an instance of Item, Form, Folder, Dataset and Document and creates a new
        ScheduleTaskDeliverable and ScheduleDeliverable and relates them to the ScheduleTask and Schedule.
        """
        return cls.execute_soa_method(
            method_name='createTaskDeliverables',
            library='ProjectManagementAw',
            service_date='2015_07',
            service_name='ScheduleManagementAw',
            params={'taskDeliverableData': taskDeliverableData},
            response_cls=CreateTaskDeliverablesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Dataset
from tcsoa.gen.Internal.DocMgmtAw._2019_12.DocMgmt import ProcessTextDatasetResponse
from tcsoa.utils import TcService


class DocMgmtService(TcService):

    @classmethod
    def processTextDataset(cls, baseObject: Dataset, content: str, action: str) -> ProcessTextDatasetResponse:
        """
        This operation extracts the contents of the text dataset to a string and returns it or saves the input string
        to the contents of a text dataset.
        
        Use cases:
        - Load the text from the dataset and return it in a string.
        - Save the input text string to the dataset.
        
        """
        return cls.execute_soa_method(
            method_name='processTextDataset',
            library='Internal-DocMgmtAw',
            service_date='2019_12',
            service_name='DocMgmt',
            params={'baseObject': baseObject, 'content': content, 'action': action},
            response_cls=ProcessTextDatasetResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.StructureManagement._2010_09.Structure import ReSequenceParameter
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class StructureService(TcService):

    @classmethod
    def resequence(cls, input: List[ReSequenceParameter]) -> ServiceData:
        """
        Re-Sequence the lines with start number and the increment number with option to re-sequence their number and
        ignoring Pert Flows.
        
        Use cases:
        User wants to re-sequence a 'WorkArea' structure, he invokes the operation with ignore flow flag, the structure
        will be re-sequenced.
        """
        return cls.execute_soa_method(
            method_name='resequence',
            library='Internal-StructureManagement',
            service_date='2010_09',
            service_name='Structure',
            params={'input': input},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, TCCalendar, Schedule, TCCalendarEvent
from tcsoa.gen.CalendarManagement._2007_06.CalendarManagement import RangeData, CalendarEventData, ModifyCalendarEvent
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CalendarContainer(TcBaseObj):
    """
    A container to hold the Calendar Events related to CalendarData
    
    :var tccalendarAttributes: The Calendar information
    :var newEvents: The event information
    """
    tccalendarAttributes: CalendarData = None
    newEvents: List[CalendarEventData] = ()


@dataclass
class CalendarData(TcBaseObj):
    """
    The information necessary to create a single TCCalendar including daily default ranges.
    
    :var name: The name of the calendar
    :var description: The description of the calendar
    :var thuRanges: The ranges for Thursday
    :var friRanges: The ranges for Friday
    :var satRanges: The ranges for Saturday
    :var timeZone: The timezone id (as specified in the Olson database eg, 'America/New_York' for US Eastern time)
    :var schedule: The schedule this calendar relates to (or null)
    :var source: The resource this calendar relates to (or null)
    :var baseCalendar: The parent BASE calendar (or null)
    :var type: The type of calendar (1-System Calendar, 2-Schedule Calendar, 3-User Calendar, 4-ScheduleMember Calendar)
    :var sunRanges: The ranges for Sunday
    :var monRanges: The ranges for Monday
    :var tueRanges: The ranges for Tuesday
    :var wedRanges: The ranges for Wednesday
    """
    name: str = ''
    description: str = ''
    thuRanges: List[RangeData] = ()
    friRanges: List[RangeData] = ()
    satRanges: List[RangeData] = ()
    timeZone: str = ''
    schedule: Schedule = None
    source: BusinessObject = None
    baseCalendar: TCCalendar = None
    type: int = 0
    sunRanges: List[RangeData] = ()
    monRanges: List[RangeData] = ()
    tueRanges: List[RangeData] = ()
    wedRanges: List[RangeData] = ()


@dataclass
class CalendarUpdate(TcBaseObj):
    """
    The information necessary to modify a calendar.
    
    :var calendar: The calendar to modify
    :var calendarAttributes: The basic calendar data to modify
    :var eventsToAdd: New calendar events to add
    :var eventsToDelete: Existing calendar events to delete
    :var eventsToUpdate: Calendar events to update.
    """
    calendar: TCCalendar = None
    calendarAttributes: CalendarData = None
    eventsToAdd: List[CalendarEventData] = ()
    eventsToDelete: List[TCCalendarEvent] = ()
    eventsToUpdate: List[ModifyCalendarEvent] = ()



from __future__ import annotations

from tcsoa.gen.Administration._2016_03.UserManagement import DeleteUsersInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class UserManagementService(TcService):

    @classmethod
    def deleteUser(cls, userInput: List[DeleteUsersInput]) -> ServiceData:
        """
        This operation deletes User objects from Teamcenter. The objects owned by the user can be deleted or keep and
        transfer their ownership to a new user. This operation requires system administration privilege.
        
        Use cases:
        Use Case 1: Delete User with given user id.
        
        Exceptions:
        >ServiceException
        
        10012 - User is not a System Administrator
        """
        return cls.execute_soa_method(
            method_name='deleteUser',
            library='Administration',
            service_date='2016_03',
            service_name='UserManagement',
            params={'userInput': userInput},
            response_cls=ServiceData,
        )



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Internal.AWS2._2015_10.FullTextSearch import PropInfo, SearchFilter2
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class Awp0ModifyFTSSavedSearchInput(TcBaseObj):
    """
    Input structure for modifySavedSearchProperties service operation.
    
    :var pinSearch: Should the saved-search object be pinned? 
    Supported values are:
    -1 = Do not pin or unpin the saved-search.
    0 = Unpin the saved-search.
    1 = Pin the saved-search.
    :var receiveNotification: Should user receive notification if saved-search results change? 
    Supported values are:
    -1 = Do not remove or set notification.
    0 = Remove notification.
    1 = Set Notification.
    :var shareSavedSearch: Should the saved search be available to all users. 
    The awp0is_global_shared property on Awp0FullTextSavedSearch object indicates if the search is shared or not. 
    -1 = Do not change the value of set awp0is_global_shared property on Awp0FullTextSavedSearch object. 
    0 = set awp0is_global_shared property on Awp0FullTextSavedSearch object to false.
    1 = set awp0is_global_shared property on Awp0FullTextSavedSearch object to true.
    :var searchFilterMap: A map of category internal names and list of filters in each category. (string/list of
    SearchFilter). Categories are defined as typeName.propertyName For example:e.g WorkspaceObject.object_type. They
    can be a Teamcenter core type/property name or classification type/property name. In some cases, it can also be a
    non-Teamcenter type/property name constructed just for the purpose of filtering on a category that may be a
    composite of other categories.
    :var propInfo: A list of PropInfo which contains information about the objects and the properties to set.
    :var chartInputParameters: A map (<string, string>) containing chart input parameters.
    NOTE: Supported keys are: ("chartOn" : the current charted on category)A map containing chart input parameters.
    
    Categories are defined as typeName.propertyName, they can be be a Teamcenter core type/property name or
    classification type/property name. In some cases, it can also be a non-Teamcenter type/property name constructed
    just for the purpose of filtering on a category that may be a composite of other categories.
    """
    pinSearch: int = 0
    receiveNotification: int = 0
    shareSavedSearch: int = 0
    searchFilterMap: SearchFilterMap3 = None
    propInfo: List[PropInfo] = ()
    chartInputParameters: ChartInputParameters = None


@dataclass
class Awp0CreateFTSSavedSearchInput(TcBaseObj):
    """
    Input structure for createFullTextSavedSearch service operation.
    
    :var pinSearch: Should the saved search object be pinned? 
    Supported values are:
    -1 = Do no pin or unpin the saved search.
    0 = Unpin the saved search.
    1 = Pin the saved search.
    :var receiveNotification: Should client receive notification if saved-search result changes? 
    Supported values are:
    -1 = Do no set or remove notification for the saved search.
    0 = Remove notification.
    1 = Set notification.
    :var shareSavedSearch: Should the saved search be available to all users? 
    The awp0is_global_shared property on Awp0FullTextSavedSearch object indicates if the search is shared or not. 
    -1 = Do not change the value of set awp0is_global_shared property on Awp0FullTextSavedSearch object. 
    0 = set awp0is_global_shared property on Awp0FullTextSavedSearch object to false.
    1 = set awp0is_global_shared property on Awp0FullTextSavedSearch object to true.
    :var savedSearchName: The saved search name. Alphanumeric and special characters are allowed.
    :var searchString: The search string. Alphanumeric and special characters are allowed.
    :var searchFilterMap: A map of category internal names and list of filters in each category. (string/list of
    SearchFilter). Categories are defined as typeName.propertyName For example:e.g WorkspaceObject.object_type. They
    can be a Teamcenter core type/property name or classification type/property name. In some cases, it can also be a
    non-Teamcenter type/property name constructed just for the purpose of filtering on a category that may be a
    composite of other categories.
    :var overwriteExistingSearch: If false and if a Awp0FullTextSavedSearch object already exists for the user having
    same name, then this operation should return a partial error. If true and if a Awp0FullTextSavedsearch object
    already exists for the user having same name then this operation deletes the existing.
    :var chartInputParameters: A map <string, string> containing chart input parameters. 
    NOTE: Supported keys are:
    - chartOn: The current charted on category internal name (format of the value: TypeName.propertyName).
    
    
    
    Categories are defined as typeName.propertyName For example:e.g WorkspaceObject.object_type. They can be a
    Teamcenter core type/property name or classification type/property name. In some cases, it can also be a
    non-Teamcenter type/property name constructed just for the purpose of filtering on a category that may be a
    composite of other categories.
    """
    pinSearch: int = 0
    receiveNotification: int = 0
    shareSavedSearch: int = 0
    savedSearchName: str = ''
    searchString: str = ''
    searchFilterMap: SearchFilterMap3 = None
    overwriteExistingSearch: bool = False
    chartInputParameters: ChartInputParameters = None


"""
A map <string, string> containing chart input parameters. 
NOTE: Supported keys are:

chartOn: The current charted on category internal name (format of the value: TypeName.propertyName)
"""
ChartInputParameters = Dict[str, str]


"""
A map of category internal names and list of filters in each category. (string/list of SearchFilter)
"""
SearchFilterMap3 = Dict[str, List[SearchFilter2]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Awb0Element, Awb0ProductContextInfo
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class OccurrenceSelection(TcBaseObj):
    """
    This operation is used to save all the occurrences selected by the user in the Active Content Experience
    application of the Active Workspace Client. The selection data is stored in the Awb0SubsetData object related to
    the Awb0BookmarkData object.
    
    :var contextProduct: The Awb0ProductContextInfo object in the context of which the selections were made in the
    Active Workspace.
    :var listOfSelectedOccurrences: List of the Awb0Element objects that were selected in the Active Workspace Client.
    """
    contextProduct: Awb0ProductContextInfo = None
    listOfSelectedOccurrences: List[Awb0Element] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import User, Person, Fnd0CustomUserProfile
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from typing import Dict, List


@dataclass
class UserObjectStructure(TcBaseObj):
    """
    The output structure for createOrUpdateUser operation. It contains created or updated User with its Person and
    Fnd0CustomUserProfile objects.
    
    :var user: The User object.
    :var person: The Person object of corresponding User.
    :var profile: The Fnd0CustomUserProfile object of corresponding User.
    """
    user: User = None
    person: Person = None
    profile: Fnd0CustomUserProfile = None


@dataclass
class CreateOrUpdateUserInputs(TcBaseObj):
    """
    Input structure to create or update User object.
    
    :var userId: User ID of the User object which need to be created or updated.
    :var person: Name of the Person this User belongs to.
    :var password: Password of the User in plain text.
    :var defaultGroup: Default group of the User.
    :var newOwner: The user ID of the new owning user of objects owned by the user to be deactivated.  It should be set
    only if the current user is going to be deactivated.
    :var newOwningGroup: The name of new owning group of objects owned by the user to be deactivated. It should be set
    only if the current user is going to be deactivated.
    :var userPropertyMap: Property map (string/list of strings) with property name of User and list of values in string
    format. The calling client is responsible for converting the different property types (int, float, date .etc) to a
    string using the appropriate functions in the SOA client framework&rsquo;s Property class.
    :var userAddlPropertyMap: Property map (string/list of strings) with property name of Fnd0CustomUserProfile and
    list of values in string format. The calling client is responsible for converting the different property types
    (int, float, date .etc) to a string using the appropriate functions in the SOA client framework&rsquo;s Property
    class.
    """
    userId: str = ''
    person: str = ''
    password: str = ''
    defaultGroup: str = ''
    newOwner: str = ''
    newOwningGroup: str = ''
    userPropertyMap: UserPropertyData = None
    userAddlPropertyMap: UserPropertyData = None


@dataclass
class CreateOrUpdateUserResponse(TcBaseObj):
    """
    Response structure of creteOrUpdateUser operation.
    
    :var userObjectMap: User Id and its corresponding UserObjectStructure which contains User, Person and
    Fnd0CustomUserProfile objects.
    :var serviceData: The Service Data with partial errors if any.
    """
    userObjectMap: UserObjectsMap = None
    serviceData: ServiceData = None


"""
Map of User Id and corersponding User objects.
"""
UserObjectsMap = Dict[str, UserObjectStructure]


"""
A map of property names and desired values (string/string) of User object.
"""
UserPropertyData = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.DocumentManagement._2018_11.PrintOrRender import ContainsRenderableFilesResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class PrintOrRenderService(TcService):

    @classmethod
    def containsRenderableFiles(cls, selectedObjects: List[BusinessObject], renderFormat: str) -> ContainsRenderableFilesResponse:
        """
        Checks to see if 'selectedObjects' contains any files that can be rendered to the 'renderFormat' using the
        Dispatcher Render Management translator.
        
        Use cases:
        This operation is intended to be called to determine the visibility of commands such as the "Generate PDF"
        one-step command.
        
        Generate PDF (one-step command): The user selects one or more Dataset or ItemRevision (or sub-type) objects in
        Active Workspace.  The 'containsRenderableFiles' operation is called by the Active Workspace client framework
        to determine if there are any files within 'selectedObjects' that can be rendered to pdf.  If any renderable
        files are found, then the "Generate PDF" command becomes visible.  If no renderable files are found, then the
        "Generate PDF" command remains hidden.
        """
        return cls.execute_soa_method(
            method_name='containsRenderableFiles',
            library='DocumentManagement',
            service_date='2018_11',
            service_name='PrintOrRender',
            params={'selectedObjects': selectedObjects, 'renderFormat': renderFormat},
            response_cls=ContainsRenderableFilesResponse,
        )

    @classmethod
    def renderFilesSubmitRequest(cls, selectedObjects: List[BusinessObject], renderFormat: str, saveRenderedFiles: bool) -> ServiceData:
        """
        Submits a Render Management Dispatcher request to render the files within the selected objects to the specified
        file format without requiring any Item Revision Document Control (IRDC) forms to be defined in Business Modeler
        Integrated Development Environment (BMIDE).
        
        Use cases:
        Generate PDF:
        The user selects one or more Dataset or ItemRevision (or sub-type) objects in Active Workspace.  The
        'containsRenderableFiles' operation is called to determine if there are any files in the selected objects that
        can be rendered to pdf.  If any renderable files are found, the "Generate PDF" one-step command becomes
        visible.  If the user clicks on the "Generate PDF" command, the 'renderFilesSubmitRequest' operation is called
        to submit a Render Management Dispatcher request to render the valid files within the selected objects to pdf. 
        When the files have been rendered, an alert notification will be sent to inform the user that the rendering is
        complete.
        """
        return cls.execute_soa_method(
            method_name='renderFilesSubmitRequest',
            library='DocumentManagement',
            service_date='2018_11',
            service_name='PrintOrRender',
            params={'selectedObjects': selectedObjects, 'renderFormat': renderFormat, 'saveRenderedFiles': saveRenderedFiles},
            response_cls=ServiceData,
        )
